from django import forms
from intecomm_form_validators.subject import HivReviewFormValidator

from ..models import HivReview
from .mixins import CrfModelFormMixin


class HivReviewForm(CrfModelFormMixin, forms.ModelForm):
    form_validator_cls = HivReviewFormValidator

    class Meta:
        model = HivReview
        fields = "__all__"



from uuid import uuid4

from django.db import models
from edc_constants.choices import YES_NO
from edc_constants.constants import NO
from edc_model.models import BaseUuidModel, HistoricalRecords
from edc_model.validators import datetime_not_future
from edc_sites.models import CurrentSiteManager, SiteModelMixin
from edc_utils import get_utcnow
from intecomm_form_validators import RECRUITING

from ..choices import GROUP_STATUS_CHOICES


class PatientGroupManager(models.Manager):
    use_in_migrations = True


class PatientGroup(SiteModelMixin, BaseUuidModel):
    group_identifier = models.CharField(max_length=36, null=True)

    group_identifier_as_pk = models.UUIDField(
        max_length=36, default=uuid4, unique=True, editable=False
    )

    report_datetime = models.DateTimeField(
        default=get_utcnow,
        validators=[datetime_not_future],
    )

    name = models.CharField(
        verbose_name="Group name",
        max_length=25,
        unique=True,
        db_index=True,
        help_text="The name may be changed at anytime as long as it is unique.",
    )

    patients = models.ManyToManyField(
        "intecomm_screening.PatientLog",
        verbose_name="Patients",
        blank=True,
    )

    hiv_patients = models.ManyToManyField(
        "intecomm_screening.PatientLog",
        verbose_name="HIV-only",
        blank=True,
        related_name="hiv_patients",
    )

    dm_patients = models.ManyToManyField(
        "intecomm_screening.PatientLog",
        verbose_name="DM-only",
        blank=True,
        related_name="dm_patients",
    )

    htn_patients = models.ManyToManyField(
        "intecomm_screening.PatientLog",
        verbose_name="HTN-only",
        blank=True,
        related_name="htn_patients",
    )

    multi_patients = models.ManyToManyField(
        "intecomm_screening.PatientLog",
        verbose_name="Multi-morbidity",
        blank=True,
        related_name="multi_patients",
    )

    status = models.CharField(
        max_length=25,
        choices=GROUP_STATUS_CHOICES,
        default=RECRUITING,
        help_text=(
            "Will revert to 'Recruiting' if changes are made to the list of selected patients"
        ),
    )

    ratio = models.DecimalField(max_digits=10, decimal_places=4, null=True)

    bypass_group_size_min = models.BooleanField(
        verbose_name="Bypass group size minimum of 14",
        default=False,
        help_text="If ticked, you must have consulted with your study coordinator first",
    )

    bypass_group_ratio = models.BooleanField(
        verbose_name="Bypass 2:1 NCD:HIV ratio",
        default=False,
        help_text="If ticked, you must have consulted with your study coordinator first",
    )

    randomize_now = models.CharField(
        verbose_name="Randomize now?", max_length=15, choices=YES_NO, default=NO
    )

    confirm_randomize_now = models.CharField(
        verbose_name="If YES, please confirm by typing the word RANDOMIZE here",
        max_length=15,
        null=True,
        blank=True,
    )

    randomized = models.BooleanField(default=False, blank=True)

    randomized_datetime = models.DateTimeField(null=True, blank=True)

    notes = models.TextField(null=True, blank=True)

    objects = PatientGroupManager()

    on_site = CurrentSiteManager()

    history = HistoricalRecords()

    def __str__(self):
        status = " (R)" if self.randomized else f"<{self.get_status_display()}>"
        return f"{self.name.upper()} {status}" if self.name else f"<new> {self.user_created}"

    def save(self, *args, **kwargs):
        if not self.id:
            self.group_identifier = str(self.group_identifier_as_pk)
        super().save(*args, **kwargs)

    class Meta(BaseUuidModel.Meta):
        verbose_name = "Patient Group"
        verbose_name_plural = "Patient Groups"



# Generated by Django 4.1.2 on 2023-01-18 22:52

from django.conf import settings
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("sites", "0002_alter_domain_unique"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("edc_action_item", "0031_auto_20220922_2236"),
        (
            "intecomm_prn",
            "0004_rename_historicaloffschedulebaseline_historicaloffschedulecomm_and_more",
        ),
    ]

    operations = [
        migrations.RenameModel(
            old_name="HistoricalOffScheduleFollowup",
            new_name="HistoricalOffScheduleInte",
        ),
        migrations.RenameModel(
            old_name="HistoricalOnScheduleFollowup",
            new_name="HistoricalOnScheduleInte",
        ),
        migrations.RenameModel(
            old_name="OffScheduleFollowup",
            new_name="OffScheduleInte",
        ),
        migrations.RenameModel(
            old_name="OnScheduleFollowup",
            new_name="OnScheduleInte",
        ),
        migrations.AlterModelOptions(
            name="historicaloffscheduleinte",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Off-schedule Facility Integrated Care",
                "verbose_name_plural": "historical Off-schedule Facility Integrated Care",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalonscheduleinte",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical On-schedule Facility Integrated Care",
                "verbose_name_plural": "historical On-schedule Facility Integrated Care",
            },
        ),
        migrations.AlterModelOptions(
            name="offscheduleinte",
            options={
                "default_permissions": ("add", "change", "delete", "view", "export", "import"),
                "get_latest_by": "modified",
                "ordering": ("-modified", "-created"),
                "verbose_name": "Off-schedule Facility Integrated Care",
                "verbose_name_plural": "Off-schedule Facility Integrated Care",
            },
        ),
        migrations.AlterModelOptions(
            name="onscheduleinte",
            options={
                "default_permissions": ("add", "change", "delete", "view", "export", "import"),
                "get_latest_by": "modified",
                "ordering": ("-modified", "-created"),
                "verbose_name": "On-schedule Facility Integrated Care",
                "verbose_name_plural": "On-schedule Facility Integrated Care",
            },
        ),
        migrations.RenameIndex(
            model_name="offscheduleinte",
            new_name="intecomm_pr_id_494e36_idx",
            old_name="intecomm_pr_id_b1c9bc_idx",
        ),
        migrations.RenameIndex(
            model_name="onscheduleinte",
            new_name="intecomm_pr_id_6be831_idx",
            old_name="intecomm_pr_id_a4d7d5_idx",
        ),
    ]



from edc_auth.get_clinic_codenames import get_clinic_codenames

clinic_codenames = get_clinic_codenames(
    "intecomm_prn",
    "intecomm_group",
    "intecomm_subject",
    "intecomm_consent",
    "intecomm_screening",
    "edc_qol",
    list_app="intecomm_lists",
)

# remove any user from deleting a patient group
clinic_codenames = [
    codename
    for codename in clinic_codenames
    if not codename.endswith("delete_patientgroup")
    and not codename.endswith("delete_patientgrouprando")
    and not codename.endswith("add_patientgrouprando")
    and codename != "intecomm_group.add_patientgroup"
    and codename != "intecomm_group.change_patientgroup"
]
screening_codenames = get_clinic_codenames(
    "intecomm_screening",
    list_app="intecomm_lists",
)
screening_codenames = [
    codename
    for codename in screening_codenames
    if not codename.endswith("delete_patientgroup")
    and not codename.endswith("delete_patientgrouprando")
    and not codename.endswith("add_patientgrouprando")
    and codename != "intecomm_group.add_patientgroup"
    and codename != "intecomm_group.change_patientgroup"
]

screening_codenames.sort()



from django.contrib import admin
from edc_he.modeladmin_mixins import HealthEconomicsHouseholdHeadModelAdminMixin

from ...admin_site import intecomm_subject_admin
from ...choices import (
    TZ_ETHNICITY_CHOICES,
    TZ_RELIGION_CHOICES,
    UG_ETHNICITY_CHOICES,
    UG_RELIGION_CHOICES,
)
from ...forms import HealthEconomicsHouseholdHeadForm
from ...models import HealthEconomicsHouseholdHead
from ..modeladmin_mixins import CrfModelAdmin


@admin.register(HealthEconomicsHouseholdHead, site=intecomm_subject_admin)
class HealthEconomicsHouseholdHeadAdmin(
    HealthEconomicsHouseholdHeadModelAdminMixin, CrfModelAdmin
):
    form = HealthEconomicsHouseholdHeadForm

    def formfield_for_choice_field(self, db_field, request, **kwargs):
        if getattr(request, "site", None):
            if db_field.name == "hoh_religion":
                if request.site.siteprofile.country == "uganda":
                    kwargs["choices"] = UG_RELIGION_CHOICES
                elif request.site.siteprofile.country == "tanzania":
                    kwargs["choices"] = TZ_RELIGION_CHOICES
            if db_field.name == "hoh_ethnicity":
                if request.site.siteprofile.country == "uganda":
                    kwargs["choices"] = UG_ETHNICITY_CHOICES
                elif request.site.siteprofile.country == "tanzania":
                    kwargs["choices"] = TZ_ETHNICITY_CHOICES
        return super().formfield_for_choice_field(db_field, request, **kwargs)



from django import forms
from edc_form_validators import FormValidatorMixin
from intecomm_form_validators import HealthTalkLogFormValidator

from ..models import HealthTalkLog


class HealthTalkLogForm(FormValidatorMixin, forms.ModelForm):
    form_validator_cls = HealthTalkLogFormValidator

    class Meta:
        model = HealthTalkLog
        fields = "__all__"



from edc_visit_schedule import VisitSchedule

from ..constants import VISIT_SCHEDULE
from .comm_schedule import comm_schedule
from .inte_schedule import inte_schedule

visit_schedule = VisitSchedule(
    name=VISIT_SCHEDULE,
    verbose_name="INTECOMM",
    offstudy_model="edc_offstudy.subjectoffstudy",
    death_report_model="intecomm_ae.deathreport",
    locator_model="edc_locator.subjectlocator",
    previous_visit_schedule=None,
)
visit_schedule.add_schedule(comm_schedule)
visit_schedule.add_schedule(inte_schedule)



from django.core.validators import MaxValueValidator, MinValueValidator
from django.db import models
from edc_constants.choices import YES_NO
from edc_glucose.model_mixins import FastingModelMixin, FbgModelMixin
from edc_model.models import BaseUuidModel
from edc_reportable.units import COPIES_PER_MILLILITER
from edc_vitals.model_mixins import BloodPressureModelMixin
from edc_vitals.models import DiastolicPressureField, SystolicPressureField, WeightField

from ..model_mixins import CrfModelMixin


class PatientHistory(
    CrfModelMixin, BloodPressureModelMixin, FbgModelMixin, FastingModelMixin, BaseUuidModel
):
    new_complaints = models.CharField(
        verbose_name="Does the patient have any new complaints on this visit?",
        max_length=15,
        choices=YES_NO,
    )

    new_complaints_detail = models.TextField(
        verbose_name="If yes, provide detail",
        max_length=250,
        null=True,
        blank=True,
    )

    weight_measured = models.CharField(
        verbose_name="Was the weight measured??",
        max_length=15,
        choices=YES_NO,
        help_text="If yes, indicate below",
    )

    weight = WeightField(
        null=True,
        blank=True,
    )

    bp_measured = models.CharField(
        verbose_name="Was the blood pressure measured?",
        max_length=15,
        choices=YES_NO,
        help_text="If yes, indicate below",
    )

    sys_blood_pressure = SystolicPressureField(
        verbose_name="Systolic pressure", null=True, blank=True
    )

    dia_blood_pressure = DiastolicPressureField(
        verbose_name="Diastolic pressure", null=True, blank=True
    )

    fbg_measured = models.CharField(
        verbose_name="Was the blood sugar measured?",
        max_length=15,
        choices=YES_NO,
        help_text="If yes, indicate below",
    )

    vl_measured = models.CharField(
        verbose_name="Was the viral load measured?",
        max_length=15,
        choices=YES_NO,
        help_text="If yes, indicate below",
    )

    vl_value = models.IntegerField(
        verbose_name="Last viral load",
        validators=[MinValueValidator(0), MaxValueValidator(999999)],
        null=True,
        blank=True,
        help_text=COPIES_PER_MILLILITER,
    )

    counseling = models.CharField(
        verbose_name="Did the patient receive health education/counselling?",
        max_length=15,
        choices=YES_NO,
    )

    out_refrral = models.CharField(
        verbose_name="Was there an Out-referral?",
        max_length=15,
        choices=YES_NO,
    )

    class Meta(CrfModelMixin.Meta, BaseUuidModel.Meta):
        verbose_name = "PatientHistory"
        verbose_name_plural = "PatientHistory"



from edc_model_wrapper import ModelWrapper


class PatientLogModelWrapper(ModelWrapper):
    model = "intecomm_screening.patientlog"
    next_url_attrs = ["patient_log_identifier"]
    next_url_name = "patient_log_listboard_url"

    @property
    def human_screening_identifier(self):
        human = None
        if self.patient_log_identifier:
            human = f"{self.patient_log_identifier[0:4]}-{self.patient_log_identifier[4:]}"
        return human or self.patient_log_identifier



from datetime import date, datetime
from zoneinfo import ZoneInfo

from django import forms
from django.conf import settings
from edc_appointment.utils import get_appointment_by_datetime
from edc_crf.modelform_mixins import CrfModelFormMixin
from edc_utils import convert_php_dateformat
from edc_visit_schedule.schedule.window import ScheduledVisitWindowError
from intecomm_form_validators.subject import NextAppointmentFormValidator

from ..models import NextAppointment


class NextAppointmentForm(CrfModelFormMixin, forms.ModelForm):
    form_validator_cls = NextAppointmentFormValidator

    def clean(self):
        cleaned_data = super().clean()
        self.validate_best_visit_code()
        return cleaned_data

    def validate_best_visit_code(self):
        if appt_date := self.cleaned_data.get("appt_date"):
            subject_visit = self.cleaned_data.get("subject_visit")
            try:
                appointment = get_appointment_by_datetime(
                    self.as_appt_datetime(appt_date),
                    subject_identifier=subject_visit.subject_identifier,
                    visit_schedule_name=subject_visit.visit_schedule.name,
                    schedule_name=subject_visit.schedule.name,
                    raise_if_in_gap=False,
                )
            except ScheduledVisitWindowError as e:
                raise forms.ValidationError({"appt_date": str(e)})
            if not appointment:
                raise forms.ValidationError(
                    {"appt_date": "Invalid. Must be within the followup period."}
                )
            elif appointment == subject_visit.appointment:
                raise forms.ValidationError(
                    {
                        "appt_date": (
                            "Invalid. Cannot be within window period of current appointment."
                        )
                    }
                )

            if self.cleaned_data.get("best_visit_code") != appointment.visit_code:
                date_format = convert_php_dateformat(settings.SHORT_DATE_FORMAT)
                raise forms.ValidationError(
                    {
                        "best_visit_code": (
                            f"Expected {appointment.visit_code} "
                            f"using {appt_date.strftime(date_format)} from above."
                        )
                    }
                )

    @staticmethod
    def as_appt_datetime(appt_date: date) -> datetime:
        return datetime(
            appt_date.year,
            appt_date.month,
            appt_date.day,
            23,
            59,
            59,
            tzinfo=ZoneInfo("UTC"),
        )

    class Meta:
        model = NextAppointment
        fields = "__all__"



# Generated by Django 4.1.7 on 2023-05-01 19:08
from django.db import migrations
from django.db.migrations import RunPython
from tqdm import tqdm

from intecomm_screening.models.patient_log import FilingIdentifier


def update_filing_identifier(apps, schema_editor):
    patientlog_cls = apps.get_model("intecomm_screening", "patientlog")
    total = patientlog_cls.objects.filter(filing_identifier__isnull=True).count()
    for obj in tqdm(
        patientlog_cls.objects.all().order_by("site_id", "created"),
        total=total,
    ):
        obj.filing_identifier = FilingIdentifier(site_id=obj.site_id).identifier
        obj.save_base(update_fields=["filing_identifier"])


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_screening", "0022_historicalpatientlog_filing_identifier_and_more"),
    ]

    operations = [RunPython(update_filing_identifier)]



from django.apps import apps as django_apps
from django.core.exceptions import ObjectDoesNotExist
from edc_constants.constants import NO
from edc_metadata.metadata_rules import PredicateCollection


class Predicates(PredicateCollection):
    app_label = "intecomm_subject"
    visit_model = "intecomm_subject.subjectvisit"

    @staticmethod
    def household_head_required(visit, **kwargs):
        """Note: this is a singleton"""
        model_cls = django_apps.get_model("intecomm_subject.healtheconomicshouseholdhead")
        try:
            model_cls.objects.get(subject_visit__subject_identifier=visit.subject_identifier)
        except ObjectDoesNotExist:
            return True
        return False

    @staticmethod
    def patient_required(visit, **kwargs):
        """Note: this is a singleton"""
        required = False
        patient_model_cls = django_apps.get_model("intecomm_subject.healtheconomicspatient")
        hoh_model_cls = django_apps.get_model("intecomm_subject.healtheconomicshouseholdhead")
        try:
            hoh_obj = hoh_model_cls.objects.get(
                subject_visit__subject_identifier=visit.subject_identifier
            )
        except ObjectDoesNotExist:
            pass
        else:
            if not patient_model_cls.objects.filter(
                subject_visit__subject_identifier=visit.subject_identifier
            ).exists():
                required = hoh_obj.hoh == NO
        return required

    @staticmethod
    def assets_required(visit, **kwargs):
        model_cls = django_apps.get_model("intecomm_subject.healtheconomicsassets")
        try:
            model_cls.objects.get(subject_visit__subject_identifier=visit.subject_identifier)
        except ObjectDoesNotExist:
            return True
        return False

    @staticmethod
    def property_required(visit, **kwargs):
        model_cls = django_apps.get_model("intecomm_subject.healtheconomicsproperty")
        try:
            model_cls.objects.get(subject_visit__subject_identifier=visit.subject_identifier)
        except ObjectDoesNotExist:
            return True
        return False

    @staticmethod
    def income_required(visit, **kwargs):
        model_cls = django_apps.get_model("intecomm_subject.healtheconomicsincome")
        try:
            model_cls.objects.get(subject_visit__subject_identifier=visit.subject_identifier)
        except ObjectDoesNotExist:
            return True
        return False



from django.contrib import admin
from edc_adverse_event.modeladmin_mixins import DeathReportTmgModelAdminMixin
from edc_model_admin.history import SimpleHistoryAdmin
from edc_sites.admin import SiteModelAdminMixin

from ..admin_site import intecomm_ae_admin
from ..forms import DeathReportTmgForm
from ..models import DeathReportTmg


@admin.register(DeathReportTmg, site=intecomm_ae_admin)
class DeathReportTmgAdmin(
    SiteModelAdminMixin, DeathReportTmgModelAdminMixin, SimpleHistoryAdmin
):
    form = DeathReportTmgForm



from django import forms
from edc_crf.modelform_mixins import CrfModelFormMixin
from intecomm_form_validators.subject import VitalsFormValidator

from ..models import Vitals


class VitalsForm(CrfModelFormMixin, forms.ModelForm):
    form_validator_cls = VitalsFormValidator

    class Meta:
        model = Vitals
        fields = "__all__"



from __future__ import annotations

from typing import TYPE_CHECKING

from django import forms
from django.core.exceptions import ObjectDoesNotExist
from django.utils.html import format_html
from django.utils.safestring import mark_safe
from edc_consent.utils import get_consent_model_cls
from edc_screening.utils import get_subject_screening_model_cls

from intecomm_screening.exceptions import AlreadyConsentedError
from intecomm_screening.utils import get_subject_consent_url

if TYPE_CHECKING:
    from intecomm_screening.models import SubjectScreening


def raise_if_subject_consent_exists(
    screening_identifier: str | None = None,
    subject_screening: SubjectScreening | None = None,
    is_modelform: bool | None = None,
) -> None:
    """Raises an exception if subject consent model instance exists."""
    screening_identifier = screening_identifier or subject_screening.screening_identifier
    try:
        subject_consent = get_consent_model_cls().objects.get(
            screening_identifier=screening_identifier
        )
    except ObjectDoesNotExist:
        pass
    else:
        if is_modelform:
            if not subject_screening:
                subject_screening = get_subject_screening_model_cls().objects.get(
                    screening_identifier=screening_identifier
                )
            consent_url = get_subject_consent_url(subject_screening=subject_screening)
            msg = format_html(
                'Not allowed. Subject has already consented. See subject <A href="{}">{}</A>',
                mark_safe(consent_url),  # nosec B308 B703
                subject_screening.subject_identifier,
            )
            raise forms.ValidationError(msg)

        else:
            raise AlreadyConsentedError(
                f"Subject has already consented. See {subject_consent.subject_identifier}. "
                f"Perhaps catch this in the form."
            )



from django.contrib import admin
from edc_action_item import ActionItemModelAdminMixin
from edc_model_admin.dashboard import ModelAdminSubjectDashboardMixin
from edc_model_admin.history import SimpleHistoryAdmin
from edc_protocol_incident.modeladmin_mixins import ProtocolIncidentModelAdminMixin
from edc_sites.admin import SiteModelAdminMixin

from ..admin_site import intecomm_prn_admin
from ..forms import ProtocolIncidentForm
from ..models import ProtocolIncident


@admin.register(ProtocolIncident, site=intecomm_prn_admin)
class ProtocolIncidentAdmin(
    SiteModelAdminMixin,
    ProtocolIncidentModelAdminMixin,
    ActionItemModelAdminMixin,
    ModelAdminSubjectDashboardMixin,
    SimpleHistoryAdmin,
):
    form = ProtocolIncidentForm



from django.db import models
from django.db.models import PROTECT
from edc_model.models import BaseUuidModel

from intecomm_screening.models import HealthFacility

from ..choices import APPT_DATE_INFO_SOURCES
from ..model_mixins import CrfModelMixin


def get_visit_code(mnth: int):
    if mnth < 10:
        return f"10{mnth*10}"
    return f"1{mnth*10}"


def visit_code_choices():
    codes = ["1000"]
    for i in range(1, 13):
        codes.append(get_visit_code(i))
    return tuple([(code, code) for code in codes])


class NextAppointment(CrfModelMixin, BaseUuidModel):
    health_facility = models.ForeignKey(
        HealthFacility,
        on_delete=PROTECT,
        null=True,
        blank=True,
    )

    appt_date = models.DateField(
        verbose_name="Next scheduled routine/facility appointment",
        null=True,
        blank=False,
        help_text="Should fall on an Integrated clinic day",
    )
    info_source = models.CharField(
        verbose_name="What is the source of this appointment date",
        max_length=15,
        choices=APPT_DATE_INFO_SOURCES,
        null=True,
        blank=False,
    )

    best_visit_code = models.CharField(
        verbose_name="Which study visit code is closest to this appointment date",
        max_length=15,
        choices=visit_code_choices(),
        null=True,
        blank=False,
        help_text=(
            "Click SAVE to let the EDC suggest. Once selected, interim appointments will "
            "be flagged as not required."
        ),
    )

    class Meta(CrfModelMixin.Meta, BaseUuidModel.Meta):
        verbose_name = "Next Appointment"
        verbose_name_plural = "Next Appointments"



from django.core.validators import MaxValueValidator, MinValueValidator
from django.db import models
from edc_crf.model_mixins import CrfModelMixin
from edc_lab.model_mixins import CrfWithRequisitionModelMixin, requisition_fk_options
from edc_lab_panel.panels import cd4_panel
from edc_model.models import BaseUuidModel
from edc_reportable import (
    CELLS_PER_MILLIMETER_CUBED,
    CELLS_PER_MILLIMETER_CUBED_DISPLAY,
)


class Cd4Result(CrfWithRequisitionModelMixin, CrfModelMixin, BaseUuidModel):
    lab_panel = cd4_panel

    requisition = models.ForeignKey(**requisition_fk_options)

    cd4_value = models.IntegerField(
        verbose_name="CD4 Result",
        validators=[MinValueValidator(0), MaxValueValidator(3000)],
        help_text=f"in {CELLS_PER_MILLIMETER_CUBED_DISPLAY}",
    )

    cd4_units = models.CharField(
        verbose_name="Units", max_length=25, default=CELLS_PER_MILLIMETER_CUBED, editable=False
    )

    class Meta(CrfWithRequisitionModelMixin.Meta, BaseUuidModel.Meta):
        verbose_name = "CD4 Result"
        verbose_name_plural = "CD4 Results"



from django.core.validators import MaxValueValidator, MinValueValidator
from django.db import models
from edc_constants.choices import LIKERT_FREQUENCY, SMOKER_STATUS_SIMPLE, YES_NO
from edc_constants.constants import NOT_APPLICABLE
from edc_model import utils as edc_model_utils
from edc_model.models import BaseUuidModel, DurationYMDField
from edc_model_fields.fields import OtherCharField

from ..choices import ALCOHOL_CONSUMPTION, EDUCATION, EMPLOYMENT_STATUS, MARITAL_STATUS
from ..model_mixins import CrfModelMixin


class OtherBaselineData(CrfModelMixin, BaseUuidModel):
    employment_status = models.CharField(
        verbose_name="What is the patient's employment status?",
        max_length=25,
        choices=EMPLOYMENT_STATUS,
    )

    employment_status_other = OtherCharField()

    education = models.CharField(
        verbose_name="How much formal education does the patient have?",
        max_length=25,
        choices=EDUCATION,
    )

    marital_status = models.CharField(
        verbose_name="Personal status?",
        max_length=25,
        choices=MARITAL_STATUS,
    )

    smoking_status = models.CharField(
        verbose_name="Which of these options describes you?",
        max_length=15,
        choices=SMOKER_STATUS_SIMPLE,
    )

    smoker_quit_ago = DurationYMDField(
        verbose_name="If you used to smoke but stopped, how long ago did you stop?",
        null=True,
        blank=True,
    )

    smoker_current_duration = DurationYMDField(
        verbose_name="If you are still smoking, how long have you been smoking?",
        null=True,
        blank=True,
    )

    smoker_quit_estimated_date = models.DateField(
        verbose_name="Estimated date smoker quit?",
        null=True,
        editable=False,
    )

    smoker_current_duration_estimated_date = models.DateField(
        verbose_name="Estimated date smoker has been smoking since?",
        null=True,
        blank=True,
    )

    alcohol = models.CharField(
        verbose_name="Do you drink alcohol?",
        max_length=15,
        choices=YES_NO,
    )

    alcohol_consumption = models.CharField(
        verbose_name="If yes, how often do you drink alcohol?",
        max_length=25,
        choices=ALCOHOL_CONSUMPTION,
        default=NOT_APPLICABLE,
    )

    activity_work = models.CharField(
        verbose_name=(
            "Does your work involve moderate or vigorous-intensity physical activity?"
        ),
        max_length=15,
        choices=YES_NO,
        help_text="See interviewer notes above.",
        null=True,
        blank=False,
    )

    activity_work_days_per_wk = models.IntegerField(
        verbose_name=(
            "If yes, how many DAYS do you do these physical activities as "
            "part of your work? (in a typical week)"
        ),
        validators=[MinValueValidator(0), MaxValueValidator(7)],
        blank=True,
        null=True,
        help_text="Number of days in a typical week",
    )

    activity_exercise_days_per_wk = models.IntegerField(
        verbose_name=(
            "How many DAYS do you do personal exercise, sports or fitness activities? "
            "(in a typical week)"
        ),
        validators=[
            MinValueValidator(0),
            MaxValueValidator(7, message="Maximum days in a week is 7"),
        ],
        blank=True,
        null=True,
        help_text="Number of days in a typical week",
    )

    activity_combined_mn_avg_day = models.IntegerField(
        verbose_name=(
            "On an average day, how many MINUTES do you spend on all physical activities "
            "combined? (moderate or vigorous-intensity work activities, exercise, sports "
            "and fitness)"
        ),
        validators=[MinValueValidator(0), MaxValueValidator(1080)],
        blank=True,
        null=True,
        help_text="Number of minutes in a typical day",
    )

    num_leafy_vegs_eaten = models.IntegerField(
        verbose_name="On average, how many green leafy vegetables do you eat in a day?",
        help_text="Number per day",
    )

    num_other_vegs_eaten = models.IntegerField(
        verbose_name="On average, how many other vegetables do you eat in a day?",
        help_text="Other vegetables like carrots, tomatoes, bell papers etc. Number per day",
    )

    num_fruits_eaten = models.IntegerField(
        verbose_name="On average, how many vegetables do you eat in a day?",
        help_text="Number per day",
    )

    adds_salt_to_food = models.CharField(
        verbose_name=(
            "Do you add salt to your food in addition to what has been added when cooking?"
        ),
        max_length=15,
        choices=LIKERT_FREQUENCY,
        null=True,
        blank=False,
    )

    def save(self, *args, **kwargs):
        if self.smoker_quit_ago:
            self.smoker_quit_estimated_date = edc_model_utils.duration_to_date(
                self.smoker_quit_ago, self.report_datetime
            )
        if self.smoker_current_duration:
            self.smoker_current_duration_estimated_date = edc_model_utils.duration_to_date(
                self.smoker_current_duration, self.report_datetime
            )
        super().save(*args, **kwargs)

    class Meta(CrfModelMixin.Meta, BaseUuidModel.Meta):
        verbose_name = "Other Baseline Data"
        verbose_name_plural = "Other Baseline Data"



# Generated by Django 4.1.2 on 2022-11-29 05:12

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_lists", "0002_delete_reasonsfortesting_and_more"),
        ("intecomm_prn", "0001_initial"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="endofstudy",
            name="delivery_date",
        ),
        migrations.RemoveField(
            model_name="endofstudy",
            name="pregnancy_date",
        ),
        migrations.RemoveField(
            model_name="endofstudy",
            name="toxicity_withdrawal_reason",
        ),
        migrations.RemoveField(
            model_name="endofstudy",
            name="toxicity_withdrawal_reason_other",
        ),
        migrations.RemoveField(
            model_name="endofstudy",
            name="transferred_consent",
        ),
        migrations.RemoveField(
            model_name="historicalendofstudy",
            name="delivery_date",
        ),
        migrations.RemoveField(
            model_name="historicalendofstudy",
            name="pregnancy_date",
        ),
        migrations.RemoveField(
            model_name="historicalendofstudy",
            name="toxicity_withdrawal_reason",
        ),
        migrations.RemoveField(
            model_name="historicalendofstudy",
            name="toxicity_withdrawal_reason_other",
        ),
        migrations.RemoveField(
            model_name="historicalendofstudy",
            name="transferred_consent",
        ),
        migrations.RemoveField(
            model_name="historicallosstofollowup",
            name="number_consecutive_missed_visits",
        ),
        migrations.RemoveField(
            model_name="losstofollowup",
            name="number_consecutive_missed_visits",
        ),
        migrations.AlterField(
            model_name="endofstudy",
            name="offstudy_reason",
            field=models.ForeignKey(
                limit_choices_to={
                    "name__in": [
                        "completed_followup",
                        "consent_withdrawal",
                        "clinical_withdrawal",
                        "LTFU",
                        "transferred",
                        "dead",
                        "OTHER",
                    ]
                },
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                to="intecomm_lists.offstudyreasons",
                verbose_name="Reason patient was terminated from the study",
            ),
        ),
        migrations.AlterField(
            model_name="historicalendofstudy",
            name="offstudy_reason",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                limit_choices_to={
                    "name__in": [
                        "completed_followup",
                        "consent_withdrawal",
                        "clinical_withdrawal",
                        "LTFU",
                        "transferred",
                        "dead",
                        "OTHER",
                    ]
                },
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="intecomm_lists.offstudyreasons",
                verbose_name="Reason patient was terminated from the study",
            ),
        ),
    ]



from edc_constants.constants import OTHER, UNKNOWN

from .constants import HOSPITAL_CLINIC

AE_TYPE = (
    ("sae", "Serious Adverse Event / Reaction"),
    ("aesi", "Adverse Event of Special Interest"),
    ("susar", "Serious Unexpected Adverse Reaction"),
)

AE_EXPECTED = (
    ("expected", "Expected"),
    ("unexpected", "Unexpected"),
)

AE_ACTION_REQUIRED = (
    ("action", "Further action is required"),
    ("no_action", "No further action is required"),
)

DEATH_LOCATIONS = (
    ("home", "At home"),
    (HOSPITAL_CLINIC, "Hospital/clinic"),
    ("elsewhere", "Elsewhere"),
)


INFORMANT_RELATIONSHIP = (
    ("husband_wife", "Husband/wife"),
    ("Parent", "Parent"),
    ("child", "Child"),
    (UNKNOWN, "Unknown"),
    (OTHER, "Other"),
)



from __future__ import annotations

from django.urls import reverse
from django_audit_fields import ModelAdminAuditFieldsMixin
from django_revision.modeladmin_mixin import ModelAdminRevisionMixin
from edc_model_admin.history import SimpleHistoryAdmin
from edc_model_admin.mixins import (
    ModelAdminBypassDefaultFormClsMixin,
    ModelAdminFormAutoNumberMixin,
    ModelAdminFormInstructionsMixin,
    ModelAdminInstitutionMixin,
    ModelAdminNextUrlRedirectMixin,
    ModelAdminRedirectAllToChangelistMixin,
    ModelAdminRedirectOnDeleteMixin,
    TemplatesModelAdminMixin,
)
from edc_notification import NotificationModelAdminMixin


class BaseModelAdminMixin(
    TemplatesModelAdminMixin,
    ModelAdminRedirectOnDeleteMixin,
    ModelAdminFormInstructionsMixin,
    ModelAdminFormAutoNumberMixin,
    ModelAdminRevisionMixin,
    ModelAdminInstitutionMixin,
    ModelAdminNextUrlRedirectMixin,
    NotificationModelAdminMixin,
    ModelAdminAuditFieldsMixin,
    ModelAdminBypassDefaultFormClsMixin,
    SimpleHistoryAdmin,
):
    show_cancel = True
    view_on_site = False
    save_on_top = True


class RedirectAllToPatientLogModelAdminMixin(ModelAdminRedirectAllToChangelistMixin):
    changelist_url = "intecomm_screening_admin:intecomm_screening_patientlog_changelist"
    change_search_field_name = "screening_identifier"
    add_search_field_name = "screening_identifier"


class ChangeListTopBarModelAdminMixin:
    changelist_top_bar_selected: str = None
    changelist_top_bar_add_url: str = None

    def changelist_view(self, request, extra_context=None):
        extra_context = extra_context or {}
        extra_context["changelist_top_bar_selected"] = self.changelist_top_bar_selected
        extra_context["changelist_top_bar_add_url"] = reverse(self.changelist_top_bar_add_url)
        return super().changelist_view(request, extra_context=extra_context)



from django.contrib import admin
from django.utils.html import format_html
from edc_sites import get_site_name
from edc_sites.admin import SiteModelAdminMixin

from intecomm_sites import all_sites

from ..admin_site import intecomm_screening_admin
from ..forms import HealthFacilityForm
from ..models import HealthFacility
from .modeladmin_mixins import BaseModelAdminMixin, ChangeListTopBarModelAdminMixin


@admin.register(HealthFacility, site=intecomm_screening_admin)
class HealthFacilityAdmin(
    SiteModelAdminMixin, ChangeListTopBarModelAdminMixin, BaseModelAdminMixin
):
    form = HealthFacilityForm
    show_object_tools = True
    change_list_template: str = "intecomm_screening/admin/healthfacility_change_list.html"
    change_list_title = HealthFacility._meta.verbose_name_plural
    change_list_note = "These are health facilites within a hospital"

    changelist_top_bar_selected = "healthfacility"
    changelist_top_bar_add_url = (
        "intecomm_screening_admin:intecomm_screening_healthfacility_add"
    )

    fieldsets = (
        (
            None,
            {
                "fields": (
                    "report_datetime",
                    "name",
                    "health_facility_type",
                    "health_facility_type_other",
                )
            },
        ),
        (
            "Clinic days",
            {"fields": ("mon", "tue", "wed", "thu", "fri", "sat")},
        ),
        (
            "Location",
            {
                "description": "Provide this information if available",
                "fields": (
                    "distance",
                    "gps",
                    "latitude",
                    "longitude",
                ),
            },
        ),
        (
            "Notes",
            {
                "fields": ("notes",),
            },
        ),
    )

    list_display = (
        "name",
        "health_facility_type",
        "clinic_days",
        "distance",
        "map",
    )

    list_filter = (
        "report_datetime",
        "health_facility_type",
    )

    radio_fields = {
        "health_facility_type": admin.VERTICAL,
    }

    search_fields = (
        "name",
        "health_facility_type__name",
    )

    @admin.display(description="Map")
    def map(self, obj=None):
        if obj.latitude and obj.longitude:
            return format_html(
                f'<A href="https://www.google.com/maps/@{obj.latitude},{obj.longitude},15z">'
                '<i class="fas fa-location-dot"></i>'
            )
        return None

    @staticmethod
    def site_name(obj=None):
        get_site_name(obj.site.id, all_sites)
        return obj.name

    @admin.display(description="Clinic Days")
    def clinic_days(self, obj=None) -> str:
        days = []
        if obj.mon:
            days.append("Mon")
        if obj.tue:
            days.append("Tue")
        if obj.wed:
            days.append("Wed")
        if obj.thu:
            days.append("Thu")
        if obj.fri:
            days.append("Fri")
        if obj.sat:
            days.append("Sat")
        return ",".join(days) if days else "unknown"



from typing import Tuple

from dateutil.relativedelta import relativedelta
from django.contrib import admin
from django.utils.html import format_html
from django_audit_fields.admin import audit_fieldset_tuple
from edc_action_item import ActionItemModelAdminMixin, action_fieldset_tuple
from edc_constants.constants import OTHER
from edc_model_admin.dashboard import ModelAdminSubjectDashboardMixin
from edc_model_admin.history import SimpleHistoryAdmin
from edc_sites.admin import SiteModelAdminMixin

from intecomm_ae.models import DeathReport
from intecomm_consent.models import SubjectConsent

from ..admin_site import intecomm_prn_admin
from ..forms import EndOfStudyForm
from ..models import EndOfStudy, LossToFollowup


@admin.register(EndOfStudy, site=intecomm_prn_admin)
class EndOfStudyAdmin(
    SiteModelAdminMixin,
    ModelAdminSubjectDashboardMixin,
    ActionItemModelAdminMixin,
    SimpleHistoryAdmin,
):
    additional_instructions = format_html(
        "Note: if the patient is <i>deceased</i>, complete form "
        "`{}` before completing this form. "
        "<BR>If the patient is </i>lost to follow up</i>, complete form "
        "`{}` before completing this form.",
        DeathReport._meta.verbose_name,
        LossToFollowup._meta.verbose_name,
    )

    form = EndOfStudyForm

    date_hierarchy = "offstudy_datetime"

    fieldsets = (
        [
            None,
            {
                "fields": (
                    "subject_identifier",
                    "offstudy_datetime",
                    "last_seen_date",
                )
            },
        ],
        [
            "Off-study reason",
            {
                "fields": (
                    "offstudy_reason",
                    "other_offstudy_reason",
                )
            },
        ],
        [
            "Dates from supporting documents (if applicable)",
            {
                "fields": (
                    "ltfu_date",
                    "transfer_date",
                    "death_date",
                )
            },
        ],
        [
            "Withdrawn on CLINICAL grounds (if applicable)",
            {
                "description": (
                    "This section is applicable if the patient was withdrawn "
                    "on clinical grounds, as indicated above"
                ),
                "fields": (
                    "clinical_withdrawal_reason",
                    "clinical_withdrawal_investigator_decision",
                    "clinical_withdrawal_reason_other",
                ),
            },
        ],
        [
            "Comment",
            {"fields": ("comment",)},
        ],
        action_fieldset_tuple,
        audit_fieldset_tuple,
    )

    radio_fields = {
        "offstudy_reason": admin.VERTICAL,
        "clinical_withdrawal_reason": admin.VERTICAL,
    }

    def get_list_display(self, request) -> Tuple[str, ...]:
        list_display = super().get_list_display(request)
        custom_fields = (
            "subject_identifier",
            "dashboard",
            "terminated",
            "last_seen",
            "months",
            "reason",
        )
        return custom_fields + tuple(
            f for f in list_display if f not in custom_fields + ("__str__",)
        )

    def get_list_filter(self, request) -> Tuple[str, ...]:
        list_filter = super().get_list_filter(request)
        custom_fields = ("offstudy_reason", "last_seen_date")
        return custom_fields + tuple(f for f in list_filter if f not in custom_fields)

    def get_search_fields(self, request) -> Tuple[str, ...]:
        search_fields = super().get_search_fields(request)
        custom_fields = ("subject_identifier",)
        return tuple(set(custom_fields + search_fields))

    @admin.display(description="terminated", ordering="offstudy_datetime")
    def terminated(self, obj):
        return obj.offstudy_datetime.date()

    @admin.display(description="last_seen", ordering="last_seen_date")
    def last_seen(self, obj):
        return obj.last_seen_date

    @admin.display(description="Offstudy reason", ordering="offstudy_reason")
    def reason(self, obj):
        return (
            obj.offstudy_reason if obj.offstudy_reason != OTHER else obj.other_offstudy_reason
        )

    @admin.display(description="M")
    def months(self, obj):
        subject_consent = SubjectConsent.objects.get(subject_identifier=obj.subject_identifier)
        return relativedelta(obj.offstudy_datetime, subject_consent.consent_datetime).months



from django import forms
from edc_form_validators import FormValidator

from ..models import Icecapa
from .mixins import CrfModelFormMixin


class IcecapaFormValidator(FormValidator):
    pass


class IcecapaForm(CrfModelFormMixin, forms.ModelForm):
    form_validator_cls = IcecapaFormValidator

    class Meta:
        model = Icecapa
        fields = "__all__"



from django import forms
from edc_constants.constants import YES
from edc_crf.crf_form_validator_mixins import CrfFormValidatorMixin
from edc_crf.modelform_mixins import CrfModelFormMixin
from edc_dx_review.form_validator_mixins import ClinicalReviewFollowupFormValidatorMixin
from edc_form_validators import FormValidator
from edc_visit_schedule.utils import raise_if_baseline

from ..models import ClinicalReview


class ClinicalReviewFormValidator(
    ClinicalReviewFollowupFormValidatorMixin,
    CrfFormValidatorMixin,
    FormValidator,
):
    def clean(self):
        raise_if_baseline(self.cleaned_data.get("subject_visit"))
        self.required_if(
            YES,
            field="health_insurance",
            field_required="health_insurance_monthly_pay",
        )
        self.required_if(
            YES,
            field="patient_club",
            field_required="patient_club_monthly_pay",
        )


class ClinicalReviewForm(CrfModelFormMixin, forms.ModelForm):
    form_validator_cls = ClinicalReviewFormValidator

    class Meta:
        model = ClinicalReview
        fields = "__all__"



from django import forms
from intecomm_form_validators.subject import ComplicationsBaselineFormValidator

from ..models import ComplicationsBaseline
from .mixins import CrfModelFormMixin


class ComplicationsBaselineForm(CrfModelFormMixin, forms.ModelForm):
    form_validator_cls = ComplicationsBaselineFormValidator

    class Meta:
        model = ComplicationsBaseline
        fields = "__all__"



from django import forms
from edc_form_validators import FormValidatorMixin
from intecomm_form_validators import PatientGroupFormValidator

from ..models import PatientGroup


class PatientGroupForm(FormValidatorMixin, forms.ModelForm):
    form_validator_cls = PatientGroupFormValidator

    def clean_hiv_patients(self):
        hiv_patients = self.cleaned_data.get("hiv_patients")
        self.raise_if_already_in_group("hiv_patients", hiv_patients)
        return hiv_patients

    def clean_htn_patients(self):
        htn_patients = self.cleaned_data.get("htn_patients")
        self.raise_if_already_in_group("htn_patients", htn_patients)
        return htn_patients

    def clean_dm_patients(self):
        dm_patients = self.cleaned_data.get("dm_patients")
        self.raise_if_already_in_group("dm_patients", dm_patients)
        return dm_patients

    def clean_multi_patients(self):
        multi_patients = self.cleaned_data.get("multi_patients")
        self.raise_if_already_in_group("multi_patients", multi_patients)
        return multi_patients

    def raise_if_already_in_group(self, attr, data):
        qs = self._meta.model.objects.filter(**{f"{attr}__in": list(data)}).exclude(
            name=self.cleaned_data.get("name")
        )
        if qs.count() >= 1:
            patients = list(data)
            for patient_group in qs:
                for patient_log in patient_group.patients.all():
                    if patient_log in patients:
                        raise forms.ValidationError(
                            f"Patient in another group. Got `{patient_log}` in "
                            f"group `{patient_group.name}`."
                        )

    class Meta:
        model = PatientGroup
        fields = "__all__"



from edc_constants.constants import YES
from edc_crf.utils import raise_if_crf_does_not_exist
from edc_dx_review.utils import raise_if_clinical_review_does_not_exist

from ...models import (
    HealthEconomicsAssets,
    HealthEconomicsHouseholdHead,
    HealthEconomicsIncome,
    HealthEconomicsProperty,
)


class HealthEconomicsModelFormMixin:
    def clean(self):
        raise_if_clinical_review_does_not_exist(self.cleaned_data.get("subject_visit"))
        self.raise_if_he_household_head_required()
        self.raise_if_he_patient_required()
        self.raise_if_he_assets_required()
        self.raise_if_he_property_required()
        return super().clean()

    @property
    def household_head(self):
        return HealthEconomicsHouseholdHead.objects.get(
            subject_visit__subject_identifier=self.subject_identifier
        )

    def raise_if_he_household_head_required(self):
        if self._meta.model != HealthEconomicsHouseholdHead:
            raise_if_crf_does_not_exist(
                self.cleaned_data.get("subject_visit"),
                model="intecomm_subject.healtheconomicshouseholdhead",
            )

    def raise_if_he_patient_required(self):
        if self.household_head.hoh != YES and self._meta.model in [
            HealthEconomicsAssets,
            HealthEconomicsIncome,
            HealthEconomicsProperty,
        ]:
            raise_if_crf_does_not_exist(
                self.cleaned_data.get("subject_visit"),
                model="intecomm_subject.healtheconomicspatient",
            )

    def raise_if_he_assets_required(self):
        if self._meta.model in [
            HealthEconomicsIncome,
            HealthEconomicsProperty,
        ]:
            raise_if_crf_does_not_exist(
                self.cleaned_data.get("subject_visit"),
                model="intecomm_subject.healtheconomicsassets",
            )

    def raise_if_he_property_required(self):
        if self._meta.model in [
            HealthEconomicsIncome,
        ]:
            raise_if_crf_does_not_exist(
                self.cleaned_data.get("subject_visit"),
                model="intecomm_subject.healtheconomicsproperty",
            )



from edc_action_item.action_with_notification import ActionWithNotification
from edc_action_item.site_action_items import site_action_items
from edc_adverse_event.constants import DEATH_REPORT_ACTION
from edc_constants.constants import HIGH_PRIORITY
from edc_ltfu.constants import LTFU_ACTION
from edc_offstudy.constants import END_OF_STUDY_ACTION
from edc_protocol_incident.action_items import (
    ProtocolIncidentAction as BaseProtocolIncidentAction,
)
from edc_transfer.action_items import SubjectTransferAction as BaseSubjectTransferAction

from intecomm_subject.constants import MISSED_VISIT_ACTION

from .constants import OFFSCHEDULE_COMM_ACTION, OFFSCHEDULE_INTE_ACTION


class OffscheduleInteAction(ActionWithNotification):
    name = OFFSCHEDULE_INTE_ACTION
    display_name = "Submit Off-Schedule (Facility)"
    notification_display_name = "Off-Schedule (Facility)"
    parent_action_names = [
        DEATH_REPORT_ACTION,
        LTFU_ACTION,
    ]
    reference_model = "intecomm_prn.offscheduleinte"
    show_link_to_changelist = True
    admin_site_name = "intecomm_prn_admin"
    priority = HIGH_PRIORITY
    singleton = True
    next_actions = [END_OF_STUDY_ACTION]


class OffscheduleCommAction(ActionWithNotification):
    name = OFFSCHEDULE_COMM_ACTION
    display_name = "Submit Off-Schedule (Community)"
    notification_display_name = "Off-Schedule (Community)"
    parent_action_names = [
        DEATH_REPORT_ACTION,
        LTFU_ACTION,
    ]
    reference_model = "intecomm_prn.offschedulecomm"
    show_link_to_changelist = True
    admin_site_name = "intecomm_prn_admin"
    priority = HIGH_PRIORITY
    singleton = True
    next_actions = [END_OF_STUDY_ACTION]


class EndOfStudyAction(ActionWithNotification):
    name = END_OF_STUDY_ACTION
    display_name = "Submit End of Study Report"
    notification_display_name = "End of Study Report"
    parent_action_names = [OFFSCHEDULE_INTE_ACTION, OFFSCHEDULE_COMM_ACTION]
    reference_model = "intecomm_prn.endofstudy"
    show_link_to_changelist = True
    admin_site_name = "intecomm_prn_admin"
    priority = HIGH_PRIORITY
    singleton = True


class LossToFollowupAction(ActionWithNotification):
    name = LTFU_ACTION
    display_name = "Submit Loss to Follow Up Report"
    notification_display_name = " Loss to Follow Up Report"
    parent_action_names = [MISSED_VISIT_ACTION]
    reference_model = "intecomm_prn.losstofollowup"
    show_link_to_changelist = True
    show_link_to_add = True
    admin_site_name = "intecomm_prn_admin"
    priority = HIGH_PRIORITY
    singleton = True


class SubjectTransferAction(BaseSubjectTransferAction):
    reference_model = "intecomm_prn.subjecttransfer"
    admin_site_name = "intecomm_prn_admin"


class ProtocolIncidentAction(BaseProtocolIncidentAction):
    reference_model = "intecomm_prn.protocolincident"
    admin_site_name = "intecomm_prn_admin"


site_action_items.register(EndOfStudyAction)
site_action_items.register(LossToFollowupAction)
site_action_items.register(OffscheduleInteAction)
site_action_items.register(OffscheduleCommAction)
site_action_items.register(ProtocolIncidentAction)
site_action_items.register(SubjectTransferAction)



from __future__ import annotations

import re
from typing import Tuple

from django.contrib import admin
from django.core.exceptions import ObjectDoesNotExist
from django.utils.html import format_html
from django_audit_fields import audit_fieldset_tuple
from edc_consent.modeladmin_mixins import PiiNamesModelAdminMixin
from edc_constants.choices import GENDER
from edc_constants.constants import UUID_PATTERN
from edc_model_admin.mixins import ModelAdminHideDeleteButtonOnCondition
from edc_screening.utils import format_reasons_ineligible
from edc_sites.admin import SiteModelAdminMixin

from intecomm_sites import all_sites

from ..admin_site import intecomm_screening_admin
from ..forms import SubjectScreeningForm
from ..models import SubjectScreening
from ..utils import get_consent_refusal_model_cls
from .modeladmin_mixins import (
    BaseModelAdminMixin,
    RedirectAllToPatientLogModelAdminMixin,
)


@admin.register(SubjectScreening, site=intecomm_screening_admin)
class SubjectScreeningAdmin(
    SiteModelAdminMixin,
    RedirectAllToPatientLogModelAdminMixin,
    ModelAdminHideDeleteButtonOnCondition,
    PiiNamesModelAdminMixin,
    BaseModelAdminMixin,
):
    form = SubjectScreeningForm
    list_per_page = 5
    autocomplete_fields = ["site"]
    show_object_tools = True
    show_cancel = True
    change_list_template: str = "intecomm_screening/admin/subjectscreening_change_list.html"

    # PiiNamesModelAdminMixin attrs
    name_fields: list[str] = ["legal_name", "familiar_name"]
    name_display_field: str = "familiar_name"
    all_sites: dict = all_sites

    # RedirectAllToPatientLogModelAdminMixin attr
    add_search_field_name = "patient_log_identifier"
    change_search_field_name = "screening_identifier"

    additional_instructions = (
        "Patients must meet ALL of the inclusion criteria and NONE of the "
        "exclusion criteria in order to proceed to the final screening stage"
    )

    fieldsets = (
        (
            None,
            {
                "fields": (
                    "report_datetime",
                    "site",
                    "patient_log_identifier",
                )
            },
        ),
        (
            "Demographics",
            {
                "description": (
                    "Please review carefully. If anything needs to be changed, do so on "
                    "the Patient Log and try again"
                ),
                "fields": (
                    "legal_name",
                    "familiar_name",
                    "initials",
                    "hospital_identifier",
                    "gender",
                    "age_in_years",
                ),
            },
        ),
        (
            "Health facility",
            {
                "fields": (
                    "in_care_6m",
                    "in_care_duration",
                )
            },
        ),
        (
            "HIV",
            {
                "fields": (
                    "hiv_dx",
                    "hiv_dx_6m",
                    "hiv_dx_ago",
                    "art_unchanged_3m",
                    "art_stable",
                    "art_adherent",
                )
            },
        ),
        (
            "Diabetes",
            {
                "fields": (
                    "dm_dx",
                    "dm_dx_6m",
                    "dm_dx_ago",
                    "dm_complications",
                )
            },
        ),
        (
            "Hypertension",
            {
                "fields": (
                    "htn_dx",
                    "htn_dx_6m",
                    "htn_dx_ago",
                    "htn_complications",
                )
            },
        ),
        (
            "Pregnancy",
            {"fields": ("pregnant",)},
        ),
        (
            "Other history",
            {
                "fields": (
                    "excluded_by_bp_history",
                    "excluded_by_gluc_history",
                    "requires_acute_care",
                )
            },
        ),
        (
            "Location",
            {
                "fields": (
                    "lives_nearby",
                    "staying_nearby_6",
                )
            },
        ),
        (
            "Blood pressure measurements",
            {
                "fields": (
                    "sys_blood_pressure_one",
                    "dia_blood_pressure_one",
                    "sys_blood_pressure_two",
                    "dia_blood_pressure_two",
                )
            },
        ),
        (
            "Other",
            {
                "fields": (
                    "consent_ability",
                    "unsuitable_for_study",
                    "reasons_unsuitable",
                    "unsuitable_agreed",
                )
            },
        ),
        (
            "Updates",
            {
                "classes": ("collapse",),
                "fields": (
                    "screening_identifier",
                    "eligible",
                    "eligibility_datetime",
                    "real_eligibility_datetime",
                    "reasons_ineligible",
                    "subject_identifier",
                ),
            },
        ),
        audit_fieldset_tuple,
    )

    list_display = (
        "screening_identifier",
        "eligibility_status",
        "demographics",
        "reasons",
        "report_datetime",
        "user_created",
        "created",
        "site_code",
    )

    list_filter = (
        "report_datetime",
        "gender",
        "eligible",
        "consented",
        "refused",
    )

    search_fields = (
        "screening_identifier",
        "subject_identifier",
        "hospital_identifier__exact",
        "initials__exact",
        "reasons_ineligible",
        "legal_name__exact",
        "familiar_name__exact",
    )

    readonly_fields = (
        "screening_identifier",
        "eligible",
        "eligibility_datetime",
        "real_eligibility_datetime",
        "reasons_ineligible",
        "subject_identifier",
    )

    radio_fields = {
        "art_adherent": admin.VERTICAL,
        "art_stable": admin.VERTICAL,
        "art_unchanged_3m": admin.VERTICAL,
        "consent_ability": admin.VERTICAL,
        "dm_complications": admin.VERTICAL,
        "dm_dx": admin.VERTICAL,
        "dm_dx_6m": admin.VERTICAL,
        "excluded_by_bp_history": admin.VERTICAL,
        "excluded_by_gluc_history": admin.VERTICAL,
        "gender": admin.VERTICAL,
        "hiv_dx": admin.VERTICAL,
        "hiv_dx_6m": admin.VERTICAL,
        "htn_complications": admin.VERTICAL,
        "htn_dx": admin.VERTICAL,
        "htn_dx_6m": admin.VERTICAL,
        "in_care_6m": admin.VERTICAL,
        "lives_nearby": admin.VERTICAL,
        "pregnant": admin.VERTICAL,
        "requires_acute_care": admin.VERTICAL,
        "selection_method": admin.VERTICAL,
        "staying_nearby_6": admin.VERTICAL,
        "unsuitable_agreed": admin.VERTICAL,
        "unsuitable_for_study": admin.VERTICAL,
    }

    def post_url_on_delete_kwargs(self, request, obj):
        return {}

    def get_readonly_fields(self, request, obj=None) -> Tuple[str, ...]:
        readonly_fields = super().get_readonly_fields(request, obj=obj)
        if obj:
            readonly_fields = readonly_fields + ("patient_log",)
        return readonly_fields

    @staticmethod
    def eligibility_status(obj=None):
        return None

    @staticmethod
    def demographics(obj=None):
        data = [
            f"{obj.get_gender_display()} {obj.age_in_years}yrs",
            f"Initials: {obj.initials.upper()}<BR>",
            f"Hospital ID: {obj.hospital_identifier}",
        ]
        return format_html("<BR>".join(data))

    def reasons(self, obj=None):
        if not obj.reasons_ineligible:
            return self.dashboard(obj)
        return format_reasons_ineligible(obj.reasons_ineligible)

    def hide_delete_button_on_condition(self, request, object_id) -> bool:
        """Hide delete button on form if subject identifier not
        set or consent_refusal exists.
        """
        obj = self.model.objects.get(id=object_id)
        try:
            consent_refusal = get_consent_refusal_model_cls().objects.get(
                screening_identifier=obj.screening_identifier
            )
        except ObjectDoesNotExist:
            consent_refusal = None
        if not re.match(UUID_PATTERN, obj.subject_identifier) or consent_refusal:
            return True
        return super().hide_delete_button_on_condition(request, object_id)

    def formfield_for_choice_field(self, db_field, request, **kwargs):
        if db_field.name == "gender":
            kwargs["choices"] = GENDER
        return super().formfield_for_choice_field(db_field, request, **kwargs)



from edc_action_item.models import ActionModelMixin
from edc_model.models import BaseUuidModel
from edc_sites.models import SiteModelMixin
from edc_visit_schedule.model_mixins import OffScheduleModelMixin

from ..constants import OFFSCHEDULE_INTE_ACTION


class OffScheduleInte(SiteModelMixin, ActionModelMixin, OffScheduleModelMixin, BaseUuidModel):
    action_name = OFFSCHEDULE_INTE_ACTION
    offschedule_compare_dates_as_datetimes = False

    class Meta(OffScheduleModelMixin.Meta, BaseUuidModel.Meta):
        verbose_name = "Off-schedule facility-based integrated care"
        verbose_name_plural = "Off-schedule facility-based integrated care"



from django import forms
from intecomm_form_validators.subject.health_economics import (
    HealthEconomicsPropertyFormValidator,
)

from ...models import HealthEconomicsProperty
from ..mixins import CrfModelFormMixin
from .modelform_mixins import HealthEconomicsModelFormMixin


class HealthEconomicsPropertyForm(
    HealthEconomicsModelFormMixin, CrfModelFormMixin, forms.ModelForm
):
    form_validator_cls = HealthEconomicsPropertyFormValidator

    class Meta:
        model = HealthEconomicsProperty
        fields = "__all__"



from .ae_followup_admin import AeFollowupAdmin
from .ae_initial_admin import AeInitialAdmin
from .ae_susar_admin import AeSusarAdmin
from .ae_tmg_admin import AeTmgAdmin
from .death_report_admin import DeathReportAdmin
from .death_report_tmg_admin import DeathReportTmgAdmin
from .death_report_tmg_second_admin import DeathReportTmgSecondAdmin



from intecomm_rando.constants import CLINIC_CONTROL

from .base_subject_listboard_view import BaseSubjectListboardView


class InteSubjectListboardView(BaseSubjectListboardView):
    listboard_template = "inte_subject_listboard_template"
    listboard_url = "inte_subject_listboard_url"
    navbar_selected_item = "inte_followup"
    search_form_url = "inte_subject_listboard_url"
    assignment = CLINIC_CONTROL
    followup_url = "followup_inte_url"



from edc_lab.model_mixins import RequisitionModelMixin
from edc_model.models import BaseUuidModel
from edc_reference.model_mixins import ReferenceModelMixin


class SubjectRequisition(RequisitionModelMixin, ReferenceModelMixin, BaseUuidModel):
    class Meta(RequisitionModelMixin.Meta, BaseUuidModel.Meta):
        pass



from django.contrib import admin
from django_audit_fields.admin import audit_fieldset_tuple
from edc_crf.admin import crf_status_fieldset_tuple

from ..admin_site import intecomm_subject_admin
from ..forms import OtherBaselineDataForm
from ..models import OtherBaselineData
from .modeladmin_mixins import CrfModelAdmin


@admin.register(OtherBaselineData, site=intecomm_subject_admin)
class OtherBaselineDataAdmin(CrfModelAdmin):
    form = OtherBaselineDataForm

    fieldsets = (
        (None, {"fields": ("subject_visit", "report_datetime")}),
        (
            "Smoking",
            {"fields": ("smoking_status", "smoker_quit_ago", "smoker_current_duration")},
        ),
        ("Alcohol", {"fields": ("alcohol", "alcohol_consumption")}),
        (
            "Diet",
            {
                "fields": (
                    "num_leafy_vegs_eaten",
                    "num_other_vegs_eaten",
                    "num_fruits_eaten",
                    "adds_salt_to_food",
                )
            },
        ),
        (
            "Physical activity: Work",
            {
                "description": (
                    "<h5>INTERVIEWER NOTES</h5>"
                    "<p>We are asking about moderate or vigorous-intensity activity at "
                    "work.</p><p>By this we mean work that causes small to large increases "
                    "in breathing or heart rate for at <b>least 10 minutes</b> continuously"
                    "</p><p>Some examples might be work that involves carrying or lifting "
                    "loads, digging or construction.</p>"
                ),
                "fields": (
                    "activity_work",
                    "activity_work_days_per_wk",
                ),
            },
        ),
        (
            "Physical activity: Personal",
            {"fields": ("activity_exercise_days_per_wk",)},
        ),
        (
            "Physical activity: Average per day",
            {"fields": ("activity_combined_mn_avg_day",)},
        ),
        (
            "Employment",
            {
                "fields": (
                    "employment_status",
                    "employment_status_other",
                )
            },
        ),
        (
            "Education",
            {"fields": ("education",)},
        ),
        (
            "Marital Status",
            {"fields": ("marital_status",)},
        ),
        crf_status_fieldset_tuple,
        audit_fieldset_tuple,
    )

    radio_fields = {
        "crf_status": admin.VERTICAL,
        "smoking_status": admin.VERTICAL,
        "alcohol": admin.VERTICAL,
        "alcohol_consumption": admin.VERTICAL,
        "employment_status": admin.VERTICAL,
        "education": admin.VERTICAL,
        "marital_status": admin.VERTICAL,
        "activity_work": admin.VERTICAL,
        "adds_salt_to_food": admin.VERTICAL,
    }



from django.db import models
from edc_constants.choices import YES_NO, YES_NO_NA
from edc_constants.constants import NOT_APPLICABLE
from edc_model import models as edc_models
from edc_model.validators import date_not_future


class ClinicalReviewModelMixin(models.Model):
    def get_best_hiv_test_date(self):
        return self.hiv_test_date or self.hiv_test_estimated_datetime

    def get_best_dm_test_date(self):
        self.dm_test_date or self.dm_test_estimated_datetime

    def get_best_htn_test_date(self):
        self.htn_test_date or self.htn_test_estimated_datetime

    @property
    def diagnoses(self):
        return dict(hiv=self.hiv_dx, htn=self.htn_dx, dm=self.dm_dx)

    @property
    def diagnoses_labels(self):
        return dict(hiv="HIV", htn="Hypertension", dm="Diabetes")

    class Meta:
        abstract = True


class ClinicalReviewBaselineHivModelMixin(models.Model):
    hiv_test = models.CharField(
        verbose_name="Has the patient ever tested for HIV infection?",
        max_length=15,
        choices=YES_NO,
    )

    hiv_test_ago = edc_models.DurationYMDField(
        verbose_name="How long ago was the patient's most recent HIV test?",
        null=True,
        blank=True,
        help_text="If positive, most recent HIV(+) test",
    )

    hiv_test_estimated_datetime = models.DateTimeField(
        null=True,
        blank=True,
        editable=False,
        help_text="calculated by the EDC using `hiv_test_ago`",
    )

    hiv_test_date = models.DateField(
        verbose_name="Date of patient's most recent HIV test?",
        validators=[date_not_future],
        null=True,
        blank=True,
    )

    hiv_dx = models.CharField(
        verbose_name="Has the patient ever tested positive for HIV infection?",
        max_length=15,
        choices=YES_NO_NA,
        default=NOT_APPLICABLE,
        help_text="If yes, complete form `HIV Initial Review`",
    )

    def save(self, *args, **kwargs):
        if self.hiv_test_ago:
            self.hiv_test_estimated_datetime = edc_models.duration_to_date(
                self.hiv_test_ago, self.report_datetime
            )
        super().save(*args, **kwargs)

    class Meta:
        abstract = True


class ClinicalReviewBaselineHtnModelMixin(models.Model):
    htn_test = models.CharField(
        verbose_name="Has the patient ever tested for Hypertension?",
        max_length=15,
        choices=YES_NO,
    )

    htn_test_ago = edc_models.DurationYMDField(
        verbose_name="If Yes, how long ago was the patient tested for Hypertension?",
        null=True,
        blank=True,
    )

    htn_test_estimated_datetime = models.DateTimeField(
        null=True,
        blank=True,
        help_text="calculated by the EDC using `htn_test_ago`",
    )

    htn_test_date = models.DateField(
        verbose_name="Date of patient's most recent Hypertension test?",
        validators=[date_not_future],
        null=True,
        blank=True,
    )

    htn_dx = models.CharField(
        verbose_name="Has the patient ever been diagnosed with Hypertension",
        max_length=15,
        choices=YES_NO_NA,
        default=NOT_APPLICABLE,
        help_text="If yes, complete form `Hypertension Initial Review`",
    )

    def save(self, *args, **kwargs):
        if self.htn_test_ago:
            self.htn_test_estimated_datetime = edc_models.duration_to_date(
                self.htn_test_ago, self.report_datetime
            )
        super().save(*args, **kwargs)

    class Meta:
        abstract = True


class ClinicalReviewBaselineDmModelMixin(models.Model):
    dm_test = models.CharField(
        verbose_name="Has the patient ever tested for Diabetes?",
        max_length=15,
        choices=YES_NO,
    )

    dm_test_ago = edc_models.DurationYMDField(
        verbose_name="If Yes, how long ago was the patient tested for Diabetes?",
        null=True,
        blank=True,
    )

    dm_test_estimated_datetime = models.DateTimeField(
        null=True,
        blank=True,
        help_text="calculated by the EDC using `dm_test_ago`",
    )

    dm_test_date = models.DateField(
        verbose_name="Date of patient's most recent Diabetes test?",
        validators=[date_not_future],
        null=True,
        blank=True,
    )

    dm_dx = models.CharField(
        verbose_name="Have you ever been diagnosed with Diabetes",
        max_length=15,
        choices=YES_NO_NA,
        default=NOT_APPLICABLE,
        help_text="If yes, complete form `Diabetes Initial Review`",
    )

    def save(self, *args, **kwargs):
        if self.dm_test_ago:
            self.dm_test_estimated_datetime = edc_models.duration_to_date(
                self.dm_test_ago, self.report_datetime
            )
        super().save(*args, **kwargs)

    class Meta:
        abstract = True



from django.contrib import admin
from edc_adherence.model_admin_mixin import MedicationAdherenceAdminMixin

from ..admin_site import intecomm_subject_admin
from ..forms import HivMedicationAdherenceForm
from ..models import HivMedicationAdherence
from .modeladmin_mixins import CrfModelAdmin


@admin.register(HivMedicationAdherence, site=intecomm_subject_admin)
class HivMedicationAdherenceAdmin(MedicationAdherenceAdminMixin, CrfModelAdmin):
    form = HivMedicationAdherenceForm



from django.contrib import admin
from django_audit_fields import audit_fieldset_tuple
from django_revision.modeladmin_mixin import ModelAdminRevisionMixin
from edc_model_admin.history import SimpleHistoryAdmin
from edc_model_admin.mixins import (
    ModelAdminFormAutoNumberMixin,
    ModelAdminFormInstructionsMixin,
    ModelAdminInstitutionMixin,
    TemplatesModelAdminMixin,
)
from edc_sites.admin import SiteModelAdminMixin

from ..admin_site import intecomm_group_admin
from ..forms import PatientGroupAppointmentForm
from ..models import PatientGroupAppointment


@admin.register(PatientGroupAppointment, site=intecomm_group_admin)
class PatientGroupAppointmentAdmin(
    SiteModelAdminMixin,
    TemplatesModelAdminMixin,
    ModelAdminFormInstructionsMixin,
    ModelAdminFormAutoNumberMixin,
    ModelAdminRevisionMixin,
    ModelAdminInstitutionMixin,
    SimpleHistoryAdmin,
):
    form = PatientGroupAppointmentForm

    limit_related_to_current_site = ["patient_group", "community_care_location"]

    show_object_tools = True
    change_list_template: str = "intecomm_group/admin/patientgroupappointment_change_list.html"

    fieldsets = (
        (
            None,
            {
                "fields": (
                    "report_datetime",
                    "patient_group",
                    "community_care_location",
                    "appt_datetime",
                    "appt_status",
                )
            },
        ),
        audit_fieldset_tuple,
    )

    radio_fields = {"appt_status": admin.VERTICAL}



from django.contrib import admin
from django.utils.html import format_html
from edc_sites.admin import SiteModelAdminMixin

from ..admin_site import intecomm_screening_admin
from ..forms import HealthTalkLogForm
from ..models import HealthTalkLog
from .modeladmin_mixins import BaseModelAdminMixin, ChangeListTopBarModelAdminMixin


@admin.register(HealthTalkLog, site=intecomm_screening_admin)
class HealthTalkLogAdmin(
    SiteModelAdminMixin, ChangeListTopBarModelAdminMixin, BaseModelAdminMixin
):
    form = HealthTalkLogForm
    show_object_tools = True
    list_per_page = 5
    autocomplete_fields = ["health_facility"]

    # TemplatesModelAdminMixin attrs
    change_list_template: str = "intecomm_screening/admin/healthtalklog_change_list.html"
    change_list_title = HealthTalkLog._meta.verbose_name

    # ChangeListTopBarModelAdminMixin attrs
    changelist_top_bar_selected = "healthtalklog"
    changelist_top_bar_add_url = (
        "intecomm_screening_admin:intecomm_screening_healthtalklog_add"
    )

    fieldsets = (
        (
            None,
            {"fields": ("report_date",)},
        ),
        (
            "Details of talk",
            {
                "fields": (
                    "health_facility",
                    "health_talk_type",
                    "health_talk_type_other",
                    "number_attended",
                    "notes",
                )
            },
        ),
    )

    list_display = (
        "report_date",
        "health_facility",
        "health_talk_type",
        "map",
        "attended",
    )

    list_filter = (
        "report_date",
        "health_talk_type",
        "health_facility",
    )

    radio_fields = {
        "health_talk_type": admin.VERTICAL,
    }

    search_fields = (
        "health_facility__name",
        "health_facility__health_facility_type__name",
        "patient_log__legal_name__exact",
        "patient_log__familiar_name__exact",
    )

    @admin.display(description="Attended", ordering="number_attended")
    def attended(self, obj=None):
        return obj.number_attended

    @admin.display(description="Map")
    def map(self, obj=None):
        if obj.health_facility.latitude and obj.health_facility.longitude:
            return format_html(
                f'<A href="https://www.google.com/maps/@{obj.health_facility.latitude},"'
                f'"{obj.health_facility.longitude},15z">'
                '<i class="fas fa-location-dot"></i>'
            )
        return None



from django import forms
from edc_consent.utils import get_consent_model_cls
from edc_constants.constants import YES
from edc_he.constants import CHF, NHIF
from edc_sites import get_sites_by_country
from edc_utils import age
from intecomm_form_validators.subject.health_economics import (
    HealthEconomicsHouseholdHeadFormValidator,
)

from intecomm_sites import all_sites

from ...models import HealthEconomicsHouseholdHead
from ..mixins import CrfModelFormMixin

# TODO: Remove NHIF and CHF from insurance Q for uganda


class HealthEconomicsHouseholdHeadForm(CrfModelFormMixin, forms.ModelForm):
    form_validator_cls = HealthEconomicsHouseholdHeadFormValidator

    def clean(self):
        return super().clean()

    def clean_hoh_gender(self):
        hoh = self.cleaned_data.get("hoh")
        hoh_gender = self.cleaned_data.get("hoh_gender")
        if hoh and hoh == YES and hoh_gender and hoh_gender != self.subject_consent.gender:
            raise forms.ValidationError(
                "Mismatch. Subject is the head of household and "
                f"is {self.subject_consent.get_gender_display().lower()}."
            )
        return hoh_gender

    def clean_hoh_age(self):
        hoh = self.cleaned_data.get("hoh")
        hoh_age = self.cleaned_data.get("hoh_age")
        if hoh and hoh == YES and hoh_age:
            if (
                hoh_age
                != age(
                    self.subject_consent.dob, self.cleaned_data.get("report_datetime")
                ).years
            ):
                age_in_years = age(
                    self.subject_consent.dob, self.cleaned_data.get("report_datetime")
                ).years
                raise forms.ValidationError(
                    "Mismatch. Subject is the head of household and "
                    f"is {age_in_years} as of this report date."
                )
        return hoh_age

    def clean_hoh_insurance(self):
        hoh_insurance = self.cleaned_data.get("hoh_insurance")
        for obj in hoh_insurance.all():
            uganda_sites = get_sites_by_country(country="uganda", all_sites=all_sites)
            if obj.name in [NHIF, CHF] and self.related_visit.site_id in [
                s.site_id for s in uganda_sites
            ]:
                raise forms.ValidationError(
                    f"Invalid select for your country (Uganda). Got `{obj.display_name}`."
                )

        return hoh_insurance

    @property
    def subject_consent(self):
        return get_consent_model_cls().objects.get(subject_identifier=self.subject_identifier)

    class Meta:
        model = HealthEconomicsHouseholdHead
        fields = "__all__"



# Generated by Django 4.1.2 on 2022-11-30 01:13

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        (
            "intecomm_consent",
            "0002_remove_subjectconsent_intecomm_co_subject_145905_idx_and_more",
        ),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicalsubjectconsent",
            name="language",
            field=models.CharField(
                choices=[
                    ["lg", "Luganda"],
                    ["ry", "Runyakitara"],
                    ["en", "English"],
                    ["sw", "Swahili"],
                    ["mas", "Maasai"],
                ],
                help_text="The language used for the consent process will also be used during data collection.",
                max_length=25,
                verbose_name="Language of consent",
            ),
        ),
        migrations.AlterField(
            model_name="subjectconsent",
            name="language",
            field=models.CharField(
                choices=[
                    ["lg", "Luganda"],
                    ["ry", "Runyakitara"],
                    ["en", "English"],
                    ["sw", "Swahili"],
                    ["mas", "Maasai"],
                ],
                help_text="The language used for the consent process will also be used during data collection.",
                max_length=25,
                verbose_name="Language of consent",
            ),
        ),
    ]



from django.db import models
from edc_model.models import BaseUuidModel
from edc_visit_tracking.constants import VISIT_MISSED_ACTION
from edc_visit_tracking.model_mixins import SubjectVisitMissedModelMixin

from intecomm_lists.models import SubjectVisitMissedReasons

from ..model_mixins import CrfWithActionModelMixin


class SubjectVisitMissed(
    SubjectVisitMissedModelMixin,
    CrfWithActionModelMixin,
    BaseUuidModel,
):
    action_name = VISIT_MISSED_ACTION

    missed_reasons = models.ManyToManyField(SubjectVisitMissedReasons, blank=True)

    class Meta(CrfWithActionModelMixin.Meta, BaseUuidModel.Meta):
        verbose_name = "Missed Visit Report"
        verbose_name_plural = "Missed Visit Report"



from edc_list_data.model_mixins import ListModelManager, ListModelMixin
from edc_model.models import HistoricalRecords
from edc_sites.models import CurrentSiteManager, SiteModelMixin


class Location(SiteModelMixin, ListModelMixin):
    on_site = CurrentSiteManager()
    history = HistoricalRecords()
    objects = ListModelManager()

    class Meta(ListModelMixin.Meta):
        verbose_name = "Location"
        verbose_name_plural = "Locations"



from typing import Tuple

from django.contrib import admin, messages
from django.core.exceptions import MultipleObjectsReturned
from edc_action_item import action_fieldset_tuple
from edc_egfr.egfr import Egfr
from edc_lab_results.admin import BloodResultsModelAdminMixin
from edc_lab_results.fieldsets import (
    BloodResultFieldset,
    calculate_egfr_drop_fieldset,
    calculate_egfr_fieldset,
)
from edc_registration.models import RegisteredSubject
from edc_utils import get_utcnow
from edc_utils.round_up import round_half_away_from_zero
from edc_visit_schedule.constants import DAY1
from edc_visit_schedule.utils import is_baseline

from ...admin_site import intecomm_subject_admin
from ...forms import BloodResultsRftForm
from ...models import BloodResultsRft, SubjectVisit
from ..modeladmin_mixins import CrfModelAdmin


@admin.register(BloodResultsRft, site=intecomm_subject_admin)
class BloodResultsRftAdmin(BloodResultsModelAdminMixin, CrfModelAdmin):
    form = BloodResultsRftForm
    actions = ["create_or_update_egfr_notification", "recalculate_egfr"]
    fieldsets = (
        *BloodResultFieldset(
            BloodResultsRft.lab_panel,
            model_cls=BloodResultsRft,
            extra_fieldsets=[
                (5, calculate_egfr_fieldset),
                (6, calculate_egfr_drop_fieldset),
                (-1, action_fieldset_tuple),
            ],
            excluded_utest_ids=["egfr", "egfr_drop"],
        ).fieldsets,
    )

    def get_readonly_fields(self, request, obj=None) -> Tuple[str, ...]:
        readonly_fields = super().get_readonly_fields(request)
        custom_fields = (
            "egfr_value",
            "egfr_units",
            "egfr_drop_value",
            "egfr_grade",
            "egfr_drop_units",
            "egfr_drop_grade",
            "summary",
        )
        return tuple(set(custom_fields + readonly_fields))

    @admin.action(permissions=["view"], description="Recalculate eGFR")
    def recalculate_egfr(self, request, queryset):
        updated = 0
        errors = 0
        for obj in queryset:
            if is_baseline(obj.related_visit):
                baseline_egfr_value = None
            else:
                baseline_visit = SubjectVisit.objects.get(
                    subject_identifier=obj.related_visit.subject_identifier,
                    visit_code=DAY1,
                    visit_code_sequence=0,
                )
                baseline_egfr_value = BloodResultsRft.objects.get(
                    subject_visit_id=baseline_visit.id
                ).egfr_value
            rs = RegisteredSubject.objects.get(
                subject_identifier=obj.related_visit.subject_identifier
            )
            egfr_options = dict(
                dob=rs.dob,
                gender=rs.gender,
                ethnicity=rs.ethnicity,
                value_threshold=45.0000,
                report_datetime=obj.report_datetime,
                baseline_egfr_value=baseline_egfr_value,
                formula_name="ckd-epi",
                reference_range_collection_name="meta",
                subject_visit=obj.related_visit,
                creatinine_units=obj.creatinine_units,
                creatinine_value=obj.creatinine_value,
                assay_datetime=obj.assay_datetime,
            )
            egfr = Egfr(percent_drop_threshold=20.0000, **egfr_options)
            if round_half_away_from_zero(obj.egfr_value, 4) != round_half_away_from_zero(
                egfr.egfr_value, 4
            ):
                obj.egfr_value = egfr.egfr_value
                obj.egfr_units = egfr.egfr_units
                obj.egfr_grade = egfr.egfr_grade
                obj.egfr_drop_value = egfr.egfr_drop_value
                obj.egfr_drop_units = egfr.egfr_drop_units
                obj.egfr_drop_grade = egfr.egfr_drop_grade
                obj.modified = get_utcnow()
                try:
                    obj.save_base()
                except MultipleObjectsReturned:
                    errors += 1
                    messages.add_message(
                        request,
                        messages.WARNING,
                        f"Unable to update {obj.subject_visit.subject_identifier} "
                        f"{obj.subject_visit.visit_code}"
                        f"{obj.subject_visit.visit_code_sequence}",
                    )
                else:
                    updated += 1
        messages.add_message(
            request,
            messages.SUCCESS,
            f"Done. Updated {updated} documents with {errors} errors.",
        )



from __future__ import annotations

from copy import deepcopy
from datetime import datetime
from uuid import uuid4
from zoneinfo import ZoneInfo

from dateutil.relativedelta import relativedelta
from django.conf import settings
from django.contrib.sites.models import Site
from edc_appointment.constants import IN_PROGRESS_APPT, INCOMPLETE_APPT
from edc_appointment.tests.appointment_test_case_mixin import AppointmentTestCaseMixin
from edc_constants.constants import DM, FEMALE, HIV, HTN, NO, NOT_APPLICABLE, YES
from edc_facility.import_holidays import import_holidays
from edc_list_data.site_list_data import site_list_data
from edc_metadata import REQUIRED
from edc_metadata.models import CrfMetadata
from edc_randomization.site_randomizers import site_randomizers
from edc_sites import add_or_update_django_sites, get_sites_by_country
from edc_visit_schedule.constants import DAY1
from edc_visit_tracking.constants import SCHEDULED
from faker import Faker
from model_bakery import baker
from model_bakery.baker import make_recipe

from intecomm_lists.models import Conditions
from intecomm_sites.sites import fqdn
from intecomm_sites.tests.site_test_case_mixin import SiteTestCaseMixin
from intecomm_subject.models import SubjectVisit

from ..models import ConsentRefusal, PatientLog, SubjectScreening

fake = Faker()
now = datetime(2019, 5, 1).astimezone(ZoneInfo("UTC"))
tomorrow = now + relativedelta(days=1)


def get_eligible_options(patient_log: PatientLog):
    hiv_dx = True if patient_log.conditions.filter(name=HIV) else False
    dm_dx = True if patient_log.conditions.filter(name=DM) else False
    htn_dx = True if patient_log.conditions.filter(name=HTN) else False
    return dict(
        report_datetime=now,
        patient_log=patient_log,
        legal_name=patient_log.legal_name,
        familiar_name=patient_log.familiar_name,
        initials=patient_log.initials,
        hospital_identifier=patient_log.hospital_identifier,
        gender=patient_log.gender,
        age_in_years=patient_log.age_in_years,
        in_care_6m=YES,
        lives_nearby=YES,
        staying_nearby_6=YES,
        pregnant=NOT_APPLICABLE,
        excluded_by_bp_history=NO,
        excluded_by_gluc_history=NO,
        requires_acute_care=NO,
        hiv_dx=YES if hiv_dx else NO,
        hiv_dx_6m=YES if hiv_dx else NOT_APPLICABLE,
        hiv_dx_ago="1y" if hiv_dx else None,
        art_unchanged_3m=YES if hiv_dx else NOT_APPLICABLE,
        art_stable=YES if hiv_dx else NOT_APPLICABLE,
        art_adherent=YES if hiv_dx else NOT_APPLICABLE,
        dm_dx=YES if dm_dx else NO,
        dm_dx_6m=YES if dm_dx else NO,
        dm_dx_ago="1y" if dm_dx else None,
        dm_complications=NO if dm_dx else NOT_APPLICABLE,
        htn_dx=YES if htn_dx else NO,
        htn_dx_6m=YES if htn_dx else NO,
        htn_dx_ago="1y" if htn_dx else None,
        htn_complications=NO if htn_dx else NOT_APPLICABLE,
        sys_blood_pressure_one=120,
        dia_blood_pressure_one=80,
        sys_blood_pressure_two=120,
        dia_blood_pressure_two=80,
        consent_ability=YES,
        unsuitable_for_study=NO,
        unsuitable_agreed=NOT_APPLICABLE,
    )


class IntecommTestCaseMixin(AppointmentTestCaseMixin, SiteTestCaseMixin):
    fqdn = fqdn

    default_sites = get_sites_by_country("tanzania")

    site_names = [s.name for s in default_sites]

    import_randomization_list = True

    sid_count_for_tests = 5

    @classmethod
    def setUpTestData(cls):
        import_holidays(test=True)
        add_or_update_django_sites(sites=get_sites_by_country("tanzania"))
        if cls.import_randomization_list:
            randomizer_cls = site_randomizers.get("default")
            randomizer_cls.import_list(
                verbose=False, sid_count_for_tests=cls.sid_count_for_tests
            )
        site_list_data.initialize()
        site_list_data.autodiscover()

    @staticmethod
    def get_patient_log(
        legal_name: str | None = None,
        familiar_name: str | None = None,
        initials: str | None = None,
        gender: str | None = None,
        age_in_years: int | None = None,
        hospital_identifier: str | None = None,
        conditions: list[Conditions] | None = None,
        **kwargs,
    ):
        opts = dict(
            legal_name=legal_name or "NAMEA AAA",
            familiar_name=familiar_name or "NAMEA",
            initials=initials or "NA",
            gender=gender or FEMALE,
            age_in_years=age_in_years or 20,
            hospital_identifier=hospital_identifier or uuid4().hex,
            contact_number="1234567890",
        )
        opts.update(**kwargs)
        patient_log = make_recipe("intecomm_screening.patientlog", **opts)
        conditions = conditions or [HIV]
        for condition in conditions:
            patient_log.conditions.add(Conditions.objects.get(name=condition))
        return patient_log

    def get_subject_screening(
        self,
        patient_log: PatientLog | None = None,
        report_datetime: datetime | None = None,
        eligibility_datetime: datetime | None = None,
        gender: str | None = None,
        age_in_years: int | None = None,
        conditions: list[Conditions] | None = None,
        patient_log_options: dict | None = None,
    ):
        patient_log_opt = dict(
            gender=gender,
            age_in_years=age_in_years,
            conditions=conditions,
        )
        patient_log_opt.update(**(patient_log_options or {}))
        patient_log = patient_log or self.get_patient_log(**patient_log_opt)
        eligible_options = deepcopy(get_eligible_options(patient_log=patient_log))
        eligible_options.update(report_datetime=report_datetime or now)
        subject_screening = SubjectScreening.objects.create(
            patient_log_identifier=patient_log.patient_log_identifier,
            user_created="erikvw",
            user_modified="erikvw",
            **eligible_options,
        )
        screening_identifier = subject_screening.screening_identifier
        self.assertEqual(subject_screening.reasons_ineligible, None)
        self.assertTrue(subject_screening.eligible)

        subject_screening = SubjectScreening.objects.get(
            screening_identifier=screening_identifier
        )
        self.assertTrue(subject_screening.eligible)

        patient_log.screening_identifier = screening_identifier
        patient_log.save()

        if eligibility_datetime:
            subject_screening.eligibility_datetime = eligibility_datetime
            subject_screening.save()
            subject_screening = SubjectScreening.objects.get(
                screening_identifier=screening_identifier
            )
        return subject_screening

    @staticmethod
    def get_subject_consent(subject_screening, consent_datetime=None, site_id=None):
        return baker.make_recipe(
            "intecomm_consent.subjectconsent",
            user_created="erikvw",
            user_modified="erikvw",
            screening_identifier=subject_screening.screening_identifier,
            legal_name=subject_screening.legal_name,
            familiar_name=subject_screening.familiar_name,
            initials=subject_screening.initials,
            gender=subject_screening.gender,
            dob=(now.date() - relativedelta(years=subject_screening.age_in_years)),
            identity=subject_screening.hospital_identifier,
            confirm_identity=subject_screening.hospital_identifier,
            site=Site.objects.get(id=site_id or settings.SITE_ID),
            consent_datetime=consent_datetime or subject_screening.report_datetime,
            version=1,
        )

    def get_subject_visit(
        self,
        visit_code: str | None = None,
        visit_code_sequence: int | None = None,
        subject_screening=None,
        subject_consent=None,
        reason: str | None = None,
        appt_datetime: datetime | None = None,
        gender: str | None = None,
        ethnicity: str | None = None,
        age_in_years: int | None = None,
        screening_datetime: datetime | None = None,
        eligibility_datetime: datetime | None = None,
    ):
        reason = reason or SCHEDULED
        subject_screening = subject_screening or self.get_subject_screening(
            gender=gender,
            age_in_years=age_in_years,
            report_datetime=screening_datetime,
            eligibility_datetime=eligibility_datetime,
        )
        subject_consent = subject_consent or self.get_subject_consent(subject_screening)
        options = dict(
            subject_identifier=subject_consent.subject_identifier,
            visit_code=visit_code or DAY1,
            visit_code_sequence=(
                visit_code_sequence if visit_code_sequence is not None else 0
            ),
            reason=reason,
        )
        if appt_datetime:
            options.update(appt_datetime=appt_datetime)
        appointment = self.get_appointment(**options)
        subject_visit = SubjectVisit(
            appointment=appointment,
            reason=SCHEDULED,
            report_datetime=appointment.appt_datetime,
        )
        subject_visit.save()
        subject_visit.refresh_from_db()
        return subject_visit

    @staticmethod
    def get_next_subject_visit(subject_visit):
        appointment = subject_visit.appointment
        appointment.appt_status = INCOMPLETE_APPT
        appointment.save()
        appointment.refresh_from_db()
        next_appointment = appointment.next_by_timepoint
        next_appointment.appt_status = IN_PROGRESS_APPT
        next_appointment.save()
        subject_visit = SubjectVisit(
            appointment=next_appointment,
            reason=SCHEDULED,
            report_datetime=next_appointment.appt_datetime,
            visit_code=next_appointment.visit_code,
            visit_code_sequence=next_appointment.visit_code_sequence,
        )
        subject_visit.save()
        subject_visit.refresh_from_db()
        return subject_visit

    @staticmethod
    def get_crf_metadata(subject_visit):
        return CrfMetadata.objects.filter(
            subject_identifier=subject_visit.subject_identifier,
            visit_code=subject_visit.visit_code,
            visit_code_sequence=subject_visit.visit_code_sequence,
            entry_status=REQUIRED,
        )

    @staticmethod
    def get_consent_refusal(screening_identifier: str):
        return ConsentRefusal.objects.create(
            user_created="jw",
            user_modified="jw",
            screening_identifier=screening_identifier,
        )



# Generated by Django 4.1.7 on 2023-04-25 02:05

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_subject", "0037_alter_bloodresultsins_fasting_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="healtheconomics",
            name="welfare_other",
            field=models.TextField(
                blank=True, max_length=250, null=True, verbose_name="If yes, please explain"
            ),
        ),
        migrations.AddField(
            model_name="historicalhealtheconomics",
            name="welfare_other",
            field=models.TextField(
                blank=True, max_length=250, null=True, verbose_name="If yes, please explain"
            ),
        ),
    ]



from django import forms
from intecomm_form_validators.subject.health_economics import (
    HealthEconomicsPatientFormValidator,
)

from ...models import HealthEconomicsPatient
from ..mixins import CrfModelFormMixin
from .modelform_mixins import HealthEconomicsModelFormMixin


class HealthEconomicsPatientForm(
    HealthEconomicsModelFormMixin, CrfModelFormMixin, forms.ModelForm
):
    form_validator_cls = HealthEconomicsPatientFormValidator

    class Meta:
        model = HealthEconomicsPatient
        fields = "__all__"



from django.db import models
from edc_adverse_event.model_mixins import DeathReportModelMixin
from edc_model.models import BaseUuidModel

from ..model_mixins import DeathReportModelMixin as MyDeathReportModelMixin


class DeathReport(MyDeathReportModelMixin, DeathReportModelMixin, BaseUuidModel):
    study_day = models.IntegerField(default=0, editable=False, help_text="not used")

    class Meta(DeathReportModelMixin.Meta, BaseUuidModel.Meta):
        pass



from django.contrib import admin
from edc_microscopy.modeladmin_mixins import MalariaTestModelAdminMixin

from ..admin_site import intecomm_subject_admin
from ..forms import MalariaTestForm
from ..models import MalariaTest
from .modeladmin_mixins import CrfModelAdmin


@admin.register(MalariaTest, site=intecomm_subject_admin)
class MalariaTestAdmin(MalariaTestModelAdminMixin, CrfModelAdmin):
    form = MalariaTestForm



from django.db import models
from edc_constants.constants import NOT_APPLICABLE

from intecomm_subject.choices import CARE_DELIVERY


class FollowupReviewModelMixin(models.Model):
    care_delivery = models.CharField(
        verbose_name="Care for this `condition` was delivered today at ...",
        max_length=25,
        choices=CARE_DELIVERY,
        default=NOT_APPLICABLE,
    )

    care_delivery_other = models.TextField(
        verbose_name="If other facility, please specify", null=True, blank=True
    )

    class Meta:
        abstract = True



from __future__ import annotations

import typing
from decimal import Decimal
from typing import Tuple

from django.urls import reverse
from edc_constants.constants import DM, HIV, HTN
from edc_randomization.site_randomizers import site_randomizers
from edc_randomization.utils import (
    SubjectNotRandomization,
    get_assignment_description_for_subject,
    get_assignment_for_subject,
)
from intecomm_rando.constants import COMM_INTERVENTION

from .exceptions import PatientGroupNotRandomized

if typing.TYPE_CHECKING:
    from .models import PatientGroup


class PatientGroupRatioError(Exception):
    pass


def verify_patient_group_ratio_raise(
    patients, raise_on_outofrange=None
) -> Tuple[int, int, Decimal | None]:
    ncd = 0.0
    hiv = 0.0
    for patient_log in patients:
        if patient_log.conditions.filter(name__in=[DM, HTN]).exclude(name__in=[HIV]).exists():
            ncd += 1.0
        elif (
            patient_log.conditions.filter(name__in=[HIV]).exclude(name__in=[DM, HTN]).exists()
        ):
            hiv += 1.0
    if not ncd or not hiv:
        ratio = 0.0
    else:
        ratio = ncd / hiv
    if raise_on_outofrange and not (2.0 <= ratio <= 2.7):
        raise PatientGroupRatioError(
            f"Ratio NDC:HIV not met. Expected at least 2:1. Got {int(ncd)}:{int(hiv)}. "
            "Perhaps catch this in the form."
        )
    ncd = int(ncd)
    hiv = int(hiv)
    ratio = Decimal(ratio)
    return ncd, hiv, ratio


def get_assignment_for_patient_group(group_identifier: str | None) -> str:
    try:
        description = get_assignment_for_subject(
            group_identifier, randomizer_name="default", identifier_fld="group_identifier"
        )
    except SubjectNotRandomization:
        raise PatientGroupNotRandomized("Group is not randomized")
    return description


def get_assignment_description_for_patient_group(group_identifier: str | None) -> str:
    try:
        description = get_assignment_description_for_subject(
            group_identifier, randomizer_name="default", identifier_fld="group_identifier"
        )
    except SubjectNotRandomization:
        raise PatientGroupNotRandomized("Group is not randomized")
    return description


def get_group_subject_dashboards_url(patient_group: PatientGroup | None) -> str | None:
    """Returns a url to the listboard of subjects in followup
    for this group.
    """
    url = None
    randomizer = site_randomizers.get("default")

    if patient_group:
        for obj in randomizer.model_cls().objects.filter(
            group_identifier=patient_group.group_identifier
        ):
            if obj.assignment == COMM_INTERVENTION:
                url = reverse("intecomm_dashboard:comm_subject_listboard_url")
            break
        if not url:
            url = reverse("intecomm_dashboard:inte_subject_listboard_url")
        return f"{url}?q={patient_group.group_identifier}"
    return None



from django.contrib import admin
from edc_he.modeladmin_mixins import HealthEconomicsAssetsModelAdminMixin

from ...admin_site import intecomm_subject_admin
from ...forms import HealthEconomicsAssetsForm
from ...models import HealthEconomicsAssets
from ..modeladmin_mixins import CrfModelAdmin


@admin.register(HealthEconomicsAssets, site=intecomm_subject_admin)
class HealthEconomicsAssetsAdmin(
    HealthEconomicsAssetsModelAdminMixin,
    CrfModelAdmin,
):
    form = HealthEconomicsAssetsForm



from django.contrib import admin
from django_audit_fields.admin import audit_fieldset_tuple
from edc_crf.admin import crf_status_fieldset_tuple

from ..admin_site import intecomm_subject_admin
from ..models import Glucose
from .modeladmin_mixins import CrfModelAdmin


@admin.register(Glucose, site=intecomm_subject_admin)
class GlucoseAdmin(CrfModelAdmin):
    # form = GlucoseForm

    fieldsets = (
        (None, {"fields": ("subject_visit", "report_datetime")}),
        (
            "Blood Sugar Measurement",
            {
                "fields": (
                    "glucose_fasting",
                    "glucose_fasting_duration_str",
                    "glucose_date",
                    "glucose_value",
                    "glucose_quantifier",
                    "glucose_units",
                ),
            },
        ),
        crf_status_fieldset_tuple,
        audit_fieldset_tuple,
    )

    radio_fields = {
        "crf_status": admin.VERTICAL,
        "glucose_units": admin.VERTICAL,
        "glucose_fasting": admin.VERTICAL,
    }



from .consent_refusal_form import ConsentRefusalForm
from .health_facility_form import HealthFacilityForm
from .health_talk_log_form import HealthTalkLogForm
from .patient_call_form import PatientCallForm
from .patient_group_form import PatientGroupForm
from .patient_log_form import PatientLogForm
from .subject_screening_form import SubjectScreeningForm



# Generated by Django 4.1.2 on 2023-01-23 00:06

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        (
            "intecomm_screening",
            "0015_remove_healthtalklog_unique_health_facility_report_date_and_more",
        ),
    ]

    operations = [
        migrations.AddField(
            model_name="historicalpatientlog",
            name="group_identifier",
            field=models.CharField(
                blank=True,
                help_text="Auto populated when group is randimized",
                max_length=25,
                null=True,
            ),
        ),
        migrations.AddField(
            model_name="patientlog",
            name="group_identifier",
            field=models.CharField(
                blank=True,
                help_text="Auto populated when group is randimized",
                max_length=25,
                null=True,
            ),
        ),
    ]



from django import forms
from edc_crf.modelform_mixins import CrfModelFormMixin
from edc_microscopy.form_validators import MalariaTestFormValidator

from ..models import MalariaTest


class MalariaTestForm(CrfModelFormMixin, forms.ModelForm):
    form_validator_cls = MalariaTestFormValidator

    class Meta:
        model = MalariaTest
        fields = "__all__"



from django.db import models
from edc_constants.constants import NO, YES
from edc_model.models import BaseUuidModel
from edc_sites.models import SiteModelMixin
from edc_utils import formatted_date

from intecomm_screening.model_mixins import PatientCallModelMixin


class PatientFollowupCall(PatientCallModelMixin, SiteModelMixin, BaseUuidModel):
    patient_log = models.ForeignKey("intecomm_screening.PatientLog", on_delete=models.PROTECT)

    def __str__(self):
        report_dt = formatted_date(self.report_datetime.date())
        call_again = "Do not call" if self.call_again == NO else "Call again"
        if self.call_again == YES:
            msg = (
                f"{report_dt}: {self.patient_log} was last called "
                f"{self.report_datetime}. {call_again}."
            )
        else:
            msg = f"{report_dt}: {self.patient_log}."
        return msg

    def natural_key(self):
        return (self.patient_log,)

    natural_key.dependencies = [
        "sites.Site",
        "intecomm_screening.patientlog",
    ]

    class Meta(BaseUuidModel.Meta):
        verbose_name = "Patient Followup Call"
        verbose_name_plural = "Patient Followup Calls"
        ordering = ["report_datetime"]



from django import forms
from intecomm_form_validators.subject import HealthEconomicsAssetsFormValidator

from ...models import HealthEconomicsAssets
from ..mixins import CrfModelFormMixin
from .modelform_mixins import HealthEconomicsModelFormMixin


class HealthEconomicsAssetsForm(
    HealthEconomicsModelFormMixin, CrfModelFormMixin, forms.ModelForm
):
    form_validator_cls = HealthEconomicsAssetsFormValidator

    class Meta:
        model = HealthEconomicsAssets
        fields = "__all__"



from django import forms
from intecomm_form_validators.subject import HtnInitialReviewFormValidator

from ..models import HtnInitialReview
from .mixins import CrfModelFormMixin


class HtnInitialReviewForm(
    CrfModelFormMixin,
    forms.ModelForm,
):
    form_validator_cls = HtnInitialReviewFormValidator

    class Meta:
        model = HtnInitialReview
        fields = "__all__"



# Generated by Django 4.2.1 on 2023-07-05 02:16

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        (
            "intecomm_consent",
            "0008_alter_historicalsubjectconsent_screening_identifier_and_more",
        ),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="subjectconsent",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "get_latest_by": "consent_datetime",
                "ordering": ("created",),
                "verbose_name": "Subject Consent",
                "verbose_name_plural": "Subject Consents",
            },
        ),
    ]



# Generated by Django 4.1.2 on 2022-12-06 23:04

from django.conf import settings
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("sites", "0002_alter_domain_unique"),
        ("edc_action_item", "0031_auto_20220922_2236"),
        ("intecomm_prn", "0003_historicalprotocolincident_reasons_withdrawn_and_more"),
    ]

    operations = [
        migrations.RenameModel(
            old_name="HistoricalOffScheduleBaseline",
            new_name="HistoricalOffScheduleComm",
        ),
        migrations.RenameModel(
            old_name="HistoricalOnScheduleBaseline",
            new_name="HistoricalOnScheduleComm",
        ),
        migrations.RenameModel(
            old_name="OffScheduleBaseline",
            new_name="OffScheduleComm",
        ),
        migrations.RenameModel(
            old_name="OnScheduleBaseline",
            new_name="OnScheduleComm",
        ),
        migrations.AlterModelOptions(
            name="historicaloffschedulecomm",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Off-schedule Community Integrated Care",
                "verbose_name_plural": "historical Off-schedule Community Integrated Care",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalonschedulecomm",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical On-schedule Community Integrated Care",
                "verbose_name_plural": "historical On-schedule Community Integrated Care",
            },
        ),
        migrations.AlterModelOptions(
            name="offschedulecomm",
            options={
                "default_permissions": ("add", "change", "delete", "view", "export", "import"),
                "get_latest_by": "modified",
                "ordering": ("-modified", "-created"),
                "verbose_name": "Off-schedule Community Integrated Care",
                "verbose_name_plural": "Off-schedule Community Integrated Care",
            },
        ),
        migrations.AlterModelOptions(
            name="onschedulecomm",
            options={
                "default_permissions": ("add", "change", "delete", "view", "export", "import"),
                "get_latest_by": "modified",
                "ordering": ("-modified", "-created"),
                "verbose_name": "On-schedule Community Integrated Care",
                "verbose_name_plural": "On-schedule Community Integrated Care",
            },
        ),
        migrations.RenameIndex(
            model_name="offschedulecomm",
            new_name="intecomm_pr_id_3f0fba_idx",
            old_name="intecomm_pr_id_829ea5_idx",
        ),
        migrations.RenameIndex(
            model_name="onschedulecomm",
            new_name="intecomm_pr_id_d4e5db_idx",
            old_name="intecomm_pr_id_52a218_idx",
        ),
    ]



from django.db import models
from edc_crf.model_mixins import SingletonCrfModelMixin
from edc_he.model_mixins import HouseholdHeadModelMixin, HouseholdModelMixin
from edc_model.models import BaseUuidModel

from ...choices import (
    TZ_ETHNICITY_CHOICES,
    TZ_RELIGION_CHOICES,
    UG_ETHNICITY_CHOICES,
    UG_RELIGION_CHOICES,
)
from ...model_mixins import CrfModelMixin


class HealthEconomicsHouseholdHead(
    SingletonCrfModelMixin,
    HouseholdHeadModelMixin,
    HouseholdModelMixin,
    CrfModelMixin,
    BaseUuidModel,
):
    hoh_religion = models.CharField(
        verbose_name="How would you describe the household head’s religious orientation?",
        max_length=25,
        choices=UG_RELIGION_CHOICES + TZ_RELIGION_CHOICES,
    )

    hoh_ethnicity = models.CharField(
        verbose_name="What is the household head’s ethnic background?",
        max_length=25,
        choices=UG_ETHNICITY_CHOICES + TZ_ETHNICITY_CHOICES,
    )

    class Meta(CrfModelMixin.Meta, BaseUuidModel.Meta):
        verbose_name = "Health Economics: Household head"
        verbose_name_plural = "Health Economics: Household head"



from .assets import HealthEconomicsAssets
from .household_head import HealthEconomicsHouseholdHead
from .income import HealthEconomicsIncome
from .patient import HealthEconomicsPatient
from .property import HealthEconomicsProperty



from __future__ import annotations

import re
from typing import List

from django.core.exceptions import MultipleObjectsReturned, ObjectDoesNotExist
from django.db.models import Q
from django.urls import reverse
from edc_listboard.views import SubjectListboardView
from intecomm_rando.constants import CLINIC_CONTROL, COMM_INTERVENTION
from intecomm_rando.models import RandomizationList

from intecomm_group.models import PatientGroup
from intecomm_group.utils import get_assignment_for_patient_group
from intecomm_screening.models import PatientLog

from ....model_wrappers import PatientLogModelWrapper as BaseModelWrapper


class PatientLogModelWrapper(BaseModelWrapper):
    model = "intecomm_screening.patientlog"
    next_url_attrs = ["subject_identifier"]
    next_url_name = "subject_dashboard_url"


class BaseSubjectListboardView(SubjectListboardView):
    listboard_model = PatientLog
    model_wrapper_cls = PatientLogModelWrapper
    search_fields = [
        "group_identifier",
        "patientgroup__name",
        "subject_identifier",
        "screening_identifier",
        "initials",
        "filing_identifier",
        "contact_number",
        "patient_log_identifier",
        "last_4_hospital_identifier",
        "last_4_contact_number",
    ]
    name_search_field: str = "legal_name"
    identity_regex: str = r"^[A-Z0-9\ ]+$"
    identity_fields = ["hospital_identifier"]
    assignment: str = None
    followup_url: str = None

    def get_context_data(self, **kwargs) -> dict:
        context = super().get_context_data(**kwargs)
        context.update(
            followup_url=reverse(self.followup_url),
            patient_group_url=self.get_patient_group_url(**kwargs),
            patient_group_appointment_url=self.get_patient_group_appointment_url(**kwargs),
            patient_group_meeting_url=self.get_patient_group_meeting_url(**kwargs),
            patient_group=self.patient_group,
            arm=self.arm,
            COMM_INTERVENTION=COMM_INTERVENTION,
            CLINIC_CONTROL=CLINIC_CONTROL,
        )
        return context

    def get_queryset(self):
        qs = super().get_queryset()
        values_list = RandomizationList.objects.values_list("group_identifier").filter(
            group_identifier__isnull=False, assignment=self.assignment
        )
        return qs.filter(group_identifier__in=values_list)

    def extra_search_options(self, search_term) -> List[Q]:
        q_objects = super().extra_search_options(search_term)
        if re.match(r"^[A-Za-z\-]+$", search_term):
            q_objects.append(
                Q(familiar_name__exact=search_term)
                | Q(patientgroup__name__icontains=search_term)
            )
        if re.match(r"^[0-9\-]+$", search_term):
            q_objects.append(Q(group_identifier__exact=search_term))
        return q_objects

    @property
    def patient_group(self) -> PatientGroup:
        try:
            obj = PatientGroup.objects.get(
                Q(group_identifier__exact=self.raw_search_term)
                | Q(name__exact=self.raw_search_term)
            )
        except (ObjectDoesNotExist, MultipleObjectsReturned):
            obj = None
        return obj

    def get_patient_group_url(self, **kwargs) -> str:
        url = reverse("intecomm_group_admin:intecomm_group_patientgroup_changelist")
        return f"{url}?q={self.raw_search_term}"

    def get_patient_group_appointment_url(self, **kwargs) -> str:
        url = reverse("intecomm_group_admin:intecomm_group_patientgroupappointment_changelist")
        return f"{url}?q={self.raw_search_term}"

    def get_patient_group_meeting_url(self, **kwargs) -> str:
        url = reverse("intecomm_group_admin:intecomm_group_patientgroupmeeting_changelist")
        return f"{url}?q={self.raw_search_term}"

    @property
    def arm(self) -> str | None:
        if self.patient_group:
            return get_assignment_for_patient_group(self.patient_group.group_identifier)
        return None



from edc_adverse_event.model_mixins import DeathReportTmgModelMixin
from edc_model.models import BaseUuidModel


class DeathReportTmg(DeathReportTmgModelMixin, BaseUuidModel):
    class Meta(DeathReportTmgModelMixin.Meta, BaseUuidModel.Meta):
        pass



from intecomm_rando.constants import COMM_INTERVENTION

from .base_subject_listboard_view import BaseSubjectListboardView


class CommSubjectListboardView(BaseSubjectListboardView):
    listboard_template = "comm_subject_listboard_template"
    listboard_url = "comm_subject_listboard_url"
    navbar_selected_item = "comm_followup"
    search_form_url = "comm_subject_listboard_url"
    listboard_fa_icon = "fas fa-users-between-lines fa-2x"
    assignment = COMM_INTERVENTION
    followup_url = "followup_comm_url"



care_delivery_fieldset_tuple = (
    "Care",
    {"fields": ("care_delivery", "care_delivery_other")},
)

treatment_pay_methods_fieldset_tuple = (
    "Other",
    {
        "fields": (
            "health_insurance",
            "health_insurance_monthly_pay",
            "patient_club",
            "patient_club_monthly_pay",
        )
    },
)



# Generated by Django 4.1.2 on 2022-11-21 21:02

from django.db import migrations
import django_crypto_fields.fields.encrypted_char_field


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_screening", "0003_site_remove_historicalpatientlog_patient_group_and_more"),
    ]

    operations = [
        migrations.RenameField(
            model_name="historicalpatientlog",
            old_name="name",
            new_name="legal_name",
        ),
        migrations.RenameField(
            model_name="patientlog",
            old_name="name",
            new_name="legal_name",
        ),
        migrations.AddField(
            model_name="historicalpatientlog",
            name="familiar_name",
            field=django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                blank=True,
                help_text=" (Encryption: RSA local)",
                max_length=71,
                verbose_name="How should we refer to you? (if we speak to you)",
            ),
        ),
        migrations.AddField(
            model_name="patientlog",
            name="familiar_name",
            field=django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                blank=True,
                help_text=" (Encryption: RSA local)",
                max_length=71,
                verbose_name="How should we refer to you? (if we speak to you)",
            ),
        ),
    ]



from django.db import models
from edc_model.models import BaseUuidModel

from intecomm_screening.models import HealthFacility


class IdenfifierFormat(BaseUuidModel):
    health_facility = models.ForeignKey(
        HealthFacility,
        on_delete=models.PROTECT,
    )

    format_name = models.CharField(max_length=50)

    format_regex = models.CharField(max_length=150)

    comment = models.TextField(null=True, blank=True)

    class Meta(BaseUuidModel.Meta):
        verbose_name = "Idenfifier format"
        verbose_name_plural = "Idenfifier formats"



# Generated by Django 4.2.1 on 2023-06-30 02:52
from uuid import uuid4

from django.db import migrations
from django.db.migrations import RunPython
from edc_utils import get_uuid
from tqdm import tqdm


def update_identifiers_from_null_to_uuid(apps, schema_editor):
    model_cls = apps.get_model("intecomm_screening.patientlog")
    qs = model_cls.objects.filter(screening_identifier__isnull=True)
    for obj in tqdm(qs, total=qs.count()):
        obj.screening_identifier = get_uuid()
        obj.save_base(update_fields=["screening_identifier"])
    qs = model_cls.objects.filter(subject_identifier__isnull=True)
    for obj in tqdm(qs, total=qs.count()):
        obj.subject_identifier = get_uuid()
        obj.save_base(update_fields=["subject_identifier"])


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_screening", "0034_alter_consentrefusal_screening_identifier_and_more"),
    ]

    operations = [RunPython(update_identifiers_from_null_to_uuid)]



from django.contrib import admin
from django_audit_fields.admin import audit_fieldset_tuple
from edc_crf.admin import crf_status_fieldset_tuple
from edc_dx import get_diagnosis_labels_prefixes

from ..admin_site import intecomm_subject_admin
from ..forms import MedicationsForm
from ..models import Medications
from .modeladmin_mixins import CrfModelAdmin


@admin.register(Medications, site=intecomm_subject_admin)
class MedicationsAdmin(CrfModelAdmin):
    form = MedicationsForm

    fieldsets = (
        (None, {"fields": ("subject_visit", "report_datetime")}),
        (
            "Prescriptions",
            {
                "fields": tuple(f"refill_{dx}" for dx in get_diagnosis_labels_prefixes()),
            },
        ),
        crf_status_fieldset_tuple,
        audit_fieldset_tuple,
    )

    radio_fields = {
        "crf_status": admin.VERTICAL,
        **{f"refill_{dx}": admin.VERTICAL for dx in get_diagnosis_labels_prefixes()},
    }



from __future__ import annotations

from django.core.exceptions import ObjectDoesNotExist
from django.urls import reverse
from edc_subject_dashboard.views import SubjectDashboardView

from intecomm_group.models import PatientGroup
from intecomm_group.utils import get_group_subject_dashboards_url
from intecomm_screening.models import PatientLog
from intecomm_subject.constants import NOT_SCHEDULED_FOR_FACILITY

from ....model_wrappers import AppointmentModelWrapper


class DashboardView(SubjectDashboardView):
    consent_model = "intecomm_consent.subjectconsent"
    navbar_selected_item = "consented_subject"
    visit_model = "intecomm_subject.subjectvisit"
    history_button_label = "Audit"

    appointment_model_wrapper_cls = AppointmentModelWrapper

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        try:
            group = PatientGroup.objects.get(
                group_identifier=context.get("consent").object.group_identifier
            )
        except ObjectDoesNotExist:
            patient_group_url = None
            group_identifier = None
            group_name = None
        else:
            group_identifier = group.group_identifier
            group_name = group.name
            patient_group_url = reverse(
                "intecomm_group_admin:intecomm_group_patientgroup_changelist"
            )
            patient_group_url = f"{patient_group_url}?q={group.name}"
        context.update(
            subject_listboard_url="screen_group_url",
            group_identifier=group_identifier,
            group_name=group_name,
            patient_group_url=patient_group_url,
            NOT_SCHEDULED_FOR_FACILITY=NOT_SCHEDULED_FOR_FACILITY,
            patient_log=self.patient_log,
            patient_log_url=self.patient_log_url,
            group_subject_dashboards_url=get_group_subject_dashboards_url(self.patient_log),
        )
        return context

    @property
    def patient_log(self):
        try:
            return PatientLog.objects.get(subject_identifier=self.subject_identifier)
        except ObjectDoesNotExist:
            return None

    @property
    def patient_log_url(self) -> str:
        patient_log_url = reverse(
            "intecomm_screening_admin:intecomm_screening_patientlog_changelist"
        )
        return f"{patient_log_url}?q={self.filing_identifier}"

    @property
    def filing_identifier(self) -> str | None:
        if self.patient_log:
            return self.patient_log.filing_identifier
        return None



from django.contrib import admin

from ..admin_site import intecomm_prn_admin
from ..forms import OffScheduleInteForm
from ..models import OffScheduleInte
from .modeladmin_mixins import OffScheduleAdmin


@admin.register(OffScheduleInte, site=intecomm_prn_admin)
class OffScheduleInteAdmin(OffScheduleAdmin):
    form = OffScheduleInteForm



from typing import Tuple

from django.apps import apps as django_apps
from django.contrib import admin
from django.urls import reverse
from django.utils.html import format_html
from edc_sites.admin import SiteModelAdminMixin

from intecomm_screening.admin.list_filters import LastApptListFilter, NextApptListFilter
from intecomm_screening.admin.modeladmin_mixins import BaseModelAdminMixin

from ..admin_site import intecomm_group_admin
from ..models import PatientFollowupCall


@admin.register(PatientFollowupCall, site=intecomm_group_admin)
class PatientFollowupCallAdmin(SiteModelAdminMixin, BaseModelAdminMixin):
    # form = PatientCallForm
    show_object_tools = False
    change_list_template: str = "intecomm_group/admin/patientfollowupcall_change_list.html"

    fieldsets = (
        (
            None,
            {
                "fields": (
                    "patient_log",
                    "report_datetime",
                )
            },
        ),
        (
            "Contact numbers",
            {
                "fields": (
                    "contact_number",
                    "alt_contact_number",
                )
            },
        ),
        (
            "Details of call",
            {
                "fields": (
                    "answered",
                    "respondent",
                    "survival_status",
                    "catchment_area",
                    "last_appt_date",
                    "next_appt_date",
                    "call_again",
                    "comment",
                )
            },
        ),
    )

    list_display = (
        "patient_log",
        "patient_log_link",
        "report_datetime",
        "answered",
        "survival_status",
        "catchment_area",
        "call_again",
        "respondent",
    )

    list_filter = (
        "report_datetime",
        LastApptListFilter,
        NextApptListFilter,
        "answered",
        "call_again",
        "respondent",
        "survival_status",
        "catchment_area",
    )

    radio_fields = {
        "answered": admin.VERTICAL,
        "respondent": admin.VERTICAL,
        "survival_status": admin.VERTICAL,
        "catchment_area": admin.VERTICAL,
        "call_again": admin.VERTICAL,
    }

    search_fields = (
        "patient_log__id",
        "patient_log__legal_name__exact",
        "patient_log__familiar_name__exact",
    )

    @admin.display(description="Patient", ordering="patient_log")
    def patient_log_link(self, obj=None):
        url = reverse("intecomm_screening_admin:intecomm_screening_patientlog_changelist")
        url = f"{url}?q={obj.patient_log.id}"
        return format_html(f'<A href="{url}">{obj.patient_log}</a>')

    @property
    def patient_log_model_cls(self):
        return django_apps.get_model("intecomm_screening.patientlog")

    def formfield_for_foreignkey(self, db_field, request, **kwargs):
        if db_field.name == "patient_log":
            if request.GET.get("patient_log"):
                kwargs["queryset"] = self.patient_log_model_cls.objects.filter(
                    id__exact=request.GET.get("patient_log", 0)
                )
            else:
                kwargs["queryset"] = self.patient_log_model_cls.objects.none()
        return super().formfield_for_foreignkey(db_field, request, **kwargs)

    def get_readonly_fields(self, request, obj=None) -> Tuple[str, ...]:
        readonly = super().get_readonly_fields(request, obj=obj)
        if obj and "patient_log" not in readonly:
            readonly = readonly + ("patient_log",)
        return readonly



from django.core.validators import MaxValueValidator, MinValueValidator
from django.db import models
from django.utils.html import format_html
from edc_lab.constants import EQ
from edc_model.models import BaseUuidModel
from edc_model.validators import date_not_future
from edc_reportable import PERCENT

from ..model_mixins import CrfModelMixin


class Hba1cResult(CrfModelMixin, BaseUuidModel):
    drawn_date = models.DateField(
        verbose_name="Specimen collection date",
        validators=[date_not_future],
    )

    result = models.DecimalField(
        verbose_name="Result",
        max_digits=4,
        decimal_places=1,
        validators=[MinValueValidator(0.0), MaxValueValidator(20.0)],
        help_text=f"in {PERCENT}",
    )

    quantifier = models.CharField(
        verbose_name=format_html("HbA1c quantifier"),
        max_length=10,
        default=EQ,
    )

    units = models.CharField(
        verbose_name="HbA1c units", max_length=15, default=PERCENT, editable=False
    )

    class Meta(CrfModelMixin.Meta, BaseUuidModel.Meta):
        verbose_name = "HbA1c Result"
        verbose_name_plural = "HbA1c Results"



from django.contrib import admin
from edc_adverse_event.modeladmin_mixins import DeathReportTmgModelAdminMixin
from edc_model_admin.history import SimpleHistoryAdmin
from edc_sites.admin import SiteModelAdminMixin

from ..admin_site import intecomm_ae_admin
from ..forms import DeathReportTmgSecondForm
from ..models import DeathReportTmgSecond


@admin.register(DeathReportTmgSecond, site=intecomm_ae_admin)
class DeathReportTmgSecondAdmin(
    SiteModelAdminMixin, DeathReportTmgModelAdminMixin, SimpleHistoryAdmin
):
    form = DeathReportTmgSecondForm



from edc_adverse_event.model_mixins import AeInitialModelMixin
from edc_model.models import BaseUuidModel


class AeInitial(AeInitialModelMixin, BaseUuidModel):
    class Meta(AeInitialModelMixin.Meta, BaseUuidModel.Meta):
        pass



from django.conf import settings
from django.db import models
from edc_adverse_event.choices import STUDY_DRUG_RELATIONSHIP
from edc_adverse_event.models import AeClassification
from edc_constants.choices import YES_NO
from edc_model import models as edc_models
from edc_model.validators import datetime_not_future
from edc_utils import get_utcnow

from ..choices import AE_TYPE


class AeReviewModelMixin(models.Model):
    ae_initial = models.ForeignKey(
        f"{settings.ADVERSE_EVENT_APP_LABEL}.aeinitial", on_delete=models.PROTECT
    )

    report_datetime = models.DateTimeField(
        verbose_name="Report date and time",
        validators=[datetime_not_future],
        default=get_utcnow,
    )

    clinical_review_datetime = models.DateTimeField(
        blank=True,
        null=True,
        validators=[datetime_not_future],
        verbose_name="Date and time of clinical review: ",
    )

    ae_classification = models.ForeignKey(
        AeClassification,
        on_delete=models.PROTECT,
        verbose_name="Adverse Event (AE) Classification",
        null=True,
        blank=False,
    )

    ae_classification_other = edc_models.OtherCharField(max_length=250, blank=True, null=True)

    ae_type = models.CharField(verbose_name="Type of event", max_length=25, choices=AE_TYPE)

    study_drug_relation = models.CharField(
        verbose_name="Relationship to study drug:",
        max_length=25,
        choices=STUDY_DRUG_RELATIONSHIP,
        null=True,
    )

    ae_expected = models.CharField(
        verbose_name="Based on the protocol, do you believe this event is expected?",
        max_length=25,
        choices=YES_NO,
        null=True,
    )

    ae_action_required = models.CharField(
        verbose_name="If unexpected, do you believe further action is required?",
        max_length=25,
        choices=YES_NO,
        null=True,
    )

    investigator_comments = models.TextField(
        blank=True, null=True, verbose_name="This Clinical Reviewer's comments:"
    )

    original_report_agreed = models.CharField(
        verbose_name="Does this Clinical Reviewer agree with the original AE report?",
        max_length=15,
        choices=YES_NO,
        blank=False,
        null=True,
        help_text="If No, explain in the narrative below",
    )

    narrative = models.TextField(verbose_name="Narrative", blank=True, null=True)

    officials_notified = models.DateTimeField(
        blank=True,
        null=True,
        validators=[datetime_not_future],
        verbose_name="Date and time regulatory authorities notified",
    )

    class Meta:
        abstract = True



from django import forms
from edc_form_validators import FormValidatorMixin
from edc_screening.utils import get_subject_screening_or_raise, is_eligible_or_raise
from intecomm_form_validators import ConsentRefusalFormValidator

from intecomm_consent.utils import raise_if_subject_consent_exists

from ..models import ConsentRefusal


class ConsentRefusalForm(FormValidatorMixin, forms.ModelForm):
    form_validator_cls = ConsentRefusalFormValidator

    def clean(self):
        cleaned_data = super().clean()
        screening_identifier = self.cleaned_data.get("screening_identifier")
        if get_subject_screening_or_raise(screening_identifier, is_modelform=True):
            is_eligible_or_raise(
                screening_identifier=screening_identifier,
                url_name="intecomm_screening_admin:intecomm_screening_patientlog_changlist",
            )
            raise_if_subject_consent_exists(
                screening_identifier=screening_identifier, is_modelform=True
            )
        return cleaned_data

    class Meta:
        model = ConsentRefusal
        fields = "__all__"
        help_texts = {"screening_identifier": "(read-only)"}
        widgets = {"screening_identifier": forms.TextInput(attrs={"readonly": "readonly"})}



# Generated by Django 4.1.7 on 2023-04-26 12:55

import _socket
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django_audit_fields.fields.hostname_modification_field
import django_audit_fields.fields.userfield
import django_audit_fields.fields.uuid_auto_field
import django_audit_fields.models.audit_model_mixin
import django_revision.revision_field
import edc_model_fields.fields.other_charfield
import edc_sites.model_mixins
import edc_utils.date
import simple_history.models
import uuid


class Migration(migrations.Migration):
    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("intecomm_lists", "0006_consentrefusalreasons_screeningrefusalreasons_and_more"),
        ("sites", "0002_alter_domain_unique"),
        ("intecomm_screening", "0019_healthfacility_fri_healthfacility_mon_and_more"),
    ]

    operations = [
        migrations.CreateModel(
            name="ConsentRefusal",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True, default=django_audit_fields.models.audit_model_mixin.utcnow
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True, default=django_audit_fields.models.audit_model_mixin.utcnow
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                (
                    "subject_identifier_as_pk",
                    models.UUIDField(default=uuid.uuid4, editable=False),
                ),
                (
                    "subject_identifier_aka",
                    models.CharField(
                        editable=False,
                        help_text="track a previously allocated identifier.",
                        max_length=50,
                        null=True,
                        verbose_name="Subject Identifier a.k.a",
                    ),
                ),
                ("subject_identifier", models.CharField(editable=False, max_length=50)),
                ("screening_identifier", models.CharField(editable=False, max_length=50)),
                (
                    "report_datetime",
                    models.DateTimeField(
                        default=edc_utils.date.get_utcnow, verbose_name="Report date and time"
                    ),
                ),
                (
                    "other_reason",
                    edc_model_fields.fields.other_charfield.OtherCharField(
                        blank=True,
                        max_length=35,
                        null=True,
                        verbose_name="If other, please specify ...",
                    ),
                ),
                (
                    "reason",
                    models.ForeignKey(
                        max_length=25,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="intecomm_lists.consentrefusalreasons",
                        verbose_name="Reason for refusal to consent",
                    ),
                ),
                (
                    "site",
                    models.ForeignKey(
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="sites.site",
                    ),
                ),
                (
                    "subject_screening",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        to="intecomm_screening.subjectscreening",
                    ),
                ),
            ],
            options={
                "verbose_name": "Consent Refusal",
                "verbose_name_plural": "Consent Refusals",
            },
            managers=[
                ("on_site", edc_sites.model_mixins.CurrentSiteManager()),
            ],
        ),
        migrations.CreateModel(
            name="HistoricalConsentRefusal",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True, default=django_audit_fields.models.audit_model_mixin.utcnow
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True, default=django_audit_fields.models.audit_model_mixin.utcnow
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        db_index=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                    ),
                ),
                (
                    "subject_identifier_as_pk",
                    models.UUIDField(default=uuid.uuid4, editable=False),
                ),
                (
                    "subject_identifier_aka",
                    models.CharField(
                        editable=False,
                        help_text="track a previously allocated identifier.",
                        max_length=50,
                        null=True,
                        verbose_name="Subject Identifier a.k.a",
                    ),
                ),
                ("subject_identifier", models.CharField(editable=False, max_length=50)),
                ("screening_identifier", models.CharField(editable=False, max_length=50)),
                (
                    "report_datetime",
                    models.DateTimeField(
                        default=edc_utils.date.get_utcnow, verbose_name="Report date and time"
                    ),
                ),
                (
                    "other_reason",
                    edc_model_fields.fields.other_charfield.OtherCharField(
                        blank=True,
                        max_length=35,
                        null=True,
                        verbose_name="If other, please specify ...",
                    ),
                ),
                (
                    "history_id",
                    models.UUIDField(
                        default=uuid.uuid4, editable=False, primary_key=True, serialize=False
                    ),
                ),
                ("history_date", models.DateTimeField(db_index=True)),
                ("history_change_reason", models.CharField(max_length=100, null=True)),
                (
                    "history_type",
                    models.CharField(
                        choices=[("+", "Created"), ("~", "Changed"), ("-", "Deleted")],
                        max_length=1,
                    ),
                ),
                (
                    "history_user",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "reason",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        max_length=25,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="intecomm_lists.consentrefusalreasons",
                        verbose_name="Reason for refusal to consent",
                    ),
                ),
                (
                    "site",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="sites.site",
                    ),
                ),
                (
                    "subject_screening",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="intecomm_screening.subjectscreening",
                    ),
                ),
            ],
            options={
                "verbose_name": "historical Consent Refusal",
                "verbose_name_plural": "historical Consent Refusals",
                "ordering": ("-history_date", "-history_id"),
                "get_latest_by": ("history_date", "history_id"),
            },
            bases=(simple_history.models.HistoricalChanges, models.Model),
        ),
        migrations.RemoveField(
            model_name="subjectrefusal",
            name="site",
        ),
        migrations.RemoveField(
            model_name="subjectrefusal",
            name="subject_screening",
        ),
        migrations.AddField(
            model_name="historicalpatientlog",
            name="screening_refusal_reason",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="intecomm_lists.screeningrefusalreasons",
                verbose_name="Reason subject unwilling to screen",
            ),
        ),
        migrations.AddField(
            model_name="historicalpatientlog",
            name="screening_refusal_reason_other",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If other, please specify ...",
            ),
        ),
        migrations.AddField(
            model_name="historicalpatientlog",
            name="willing_to_screen",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("tbd", "To be determined")],
                default="tbd",
                help_text="If NO, explain below",
                max_length=15,
                verbose_name="Has the patient agreed to be screened for the INTECOMM study",
            ),
        ),
        migrations.AddField(
            model_name="patientlog",
            name="screening_refusal_reason",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                to="intecomm_lists.screeningrefusalreasons",
                verbose_name="Reason subject unwilling to screen",
            ),
        ),
        migrations.AddField(
            model_name="patientlog",
            name="screening_refusal_reason_other",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If other, please specify ...",
            ),
        ),
        migrations.AddField(
            model_name="patientlog",
            name="willing_to_screen",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("tbd", "To be determined")],
                default="tbd",
                help_text="If NO, explain below",
                max_length=15,
                verbose_name="Has the patient agreed to be screened for the INTECOMM study",
            ),
        ),
        migrations.DeleteModel(
            name="HistoricalSubjectRefusal",
        ),
        migrations.DeleteModel(
            name="SubjectRefusal",
        ),
    ]



from edc_adverse_event.navbars import ae_navbar_item, tmg_navbar_item
from edc_data_manager.navbar_item import dm_navbar_item
from edc_navbar import Navbar, NavbarItem, site_navbars
from edc_review_dashboard.navbars import navbar_item as review_navbar_item

no_url_namespace = False  # True if settings.APP_NAME == "intecomm_dashboard" else False

navbar = Navbar(name="intecomm_dashboard")


navbar.append_item(
    NavbarItem(
        name="screen_group",
        title="Screen/Group",
        label="Screen/Group",
        fa_icon="fa-solid fa-user-plus",
        codename="edc_screening.view_screening_listboard",
        url_name="screen_group_url",
        no_url_namespace=no_url_namespace,
    )
)

navbar.append_item(
    NavbarItem(
        name="comm_followup",
        title="Community",
        label="Community",
        fa_icon="fa-solid fa-users-between-lines",
        codename="edc_subject_dashboard.view_subject_listboard",
        url_name="followup_comm_url",
        no_url_namespace=no_url_namespace,
    )
)

navbar.append_item(
    NavbarItem(
        name="inte_followup",
        title="Facility",
        label="Facility",
        fa_icon="fa-solid fa-user-circle",
        codename="edc_subject_dashboard.view_subject_listboard",
        url_name="followup_inte_url",
        no_url_namespace=no_url_namespace,
    )
)

# navbar.append_item(
#     NavbarItem(
#         name="consented_subject",
#         title="Baseline",
#         label="Baseline",
#         fa_icon="fa-solid fa-user-circle",
#         codename="edc_subject_dashboard.view_subject_listboard",
#         url_name="subject_listboard_url",
#         no_url_namespace=no_url_namespace,
#     )
# )

navbar.append_item(review_navbar_item)
navbar.append_item(tmg_navbar_item)
navbar.append_item(ae_navbar_item)
navbar.append_item(dm_navbar_item)

site_navbars.register(navbar)



from django.contrib import admin
from django_audit_fields import audit_fieldset_tuple
from edc_crf.admin import crf_status_fieldset_tuple

from intecomm_screening.models import HealthFacility

from ..admin_site import intecomm_subject_admin
from ..constants import INTEGRATED
from ..forms import NextAppointmentForm
from ..models import NextAppointment
from .modeladmin_mixins import CrfModelAdmin


@admin.register(NextAppointment, site=intecomm_subject_admin)
class NextAppointmentAdmin(CrfModelAdmin):
    form = NextAppointmentForm
    fieldsets = (
        (None, {"fields": ("subject_visit", "report_datetime")}),
        (
            "Integrated Clinic",
            {"fields": ("health_facility", "appt_date", "info_source", "best_visit_code")},
        ),
        crf_status_fieldset_tuple,
        audit_fieldset_tuple,
    )

    radio_fields = {
        "crf_status": admin.VERTICAL,
        "info_source": admin.VERTICAL,
    }

    def formfield_for_foreignkey(self, db_field, request, **kwargs):
        if db_field.name == "health_facility":
            kwargs["queryset"] = HealthFacility.on_site.filter(
                health_facility_type__name=INTEGRATED
            )
        return super().formfield_for_foreignkey(db_field, request, **kwargs)
